#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.site.zmi.link module

"""

from uuid import uuid4

from zope.container.interfaces import IContainer
from zope.interface import Interface, implementer
from zope.intid import IIntIds
from zope.schema import Int

from pyams_content.interfaces import CREATE_CONTENT_PERMISSION, MANAGE_CONTENT_PERMISSION
from pyams_content.shared.site.interfaces import IExternalSiteLink, IInternalSiteLink, \
    ISiteContainer, ISiteLink, ISiteManager
from pyams_content.shared.site.zmi.interfaces import ISiteTreeTable
from pyams_content.shared.site.zmi.tree import SiteContainerTreeTable
from pyams_content.shared.site.zmi.widget.folder import SiteManagerFoldersSelectorFieldWidget
from pyams_content.zmi.interfaces import IDashboardColumn, IDashboardContentLabel, \
    IDashboardContentModifier, IDashboardContentNumber, IDashboardContentOwner, \
    IDashboardContentStatus, IDashboardContentStatusDatetime, IDashboardContentType, \
    IDashboardContentVersion, IDashboardContentVisibility
from pyams_skin.interfaces.view import IModalEditForm
from pyams_utils.interfaces import MISSING_INFO
from pyams_zmi.interfaces.form import IFormTitle, IPropertiesEditForm
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.form import apply_changes
from pyams_form.interfaces.form import IAJAXFormRenderer, IGroup
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.viewlet.menu import MenuDivider, MenuItem
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config, query_adapter
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_workflow.interfaces import IWorkflowPublicationInfo
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm, FormGroupChecker
from pyams_zmi.helper.event import get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager
from pyams_zmi.table import TableElementEditor
from pyams_zmi.utils import get_object_label


__docformat__ = 'restructuredtext'

from pyams_content import _


class SiteLinkAddForm(AdminModalAddForm):
    """Base site link add form"""

    subtitle = _("New link")

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        if 'parent' in self.widgets:
            self.widgets['parent'].permission = CREATE_CONTENT_PERMISSION

    def add(self, obj):
        setattr(self, '_v_target', obj.__parent__)

    def update_content(self, link, data):
        data = data.get(self, data)
        # locate new link
        intids = get_utility(IIntIds)
        parent = intids.queryObject(data.pop('parent'))
        if parent is not None:
            uuid = str(uuid4())
            parent[uuid] = link
        # initialize new link attributes
        apply_changes(self, link, data)


@adapter_config(required=(ISiteContainer, IAdminLayer, SiteLinkAddForm),
                provides=IFormTitle)
def site_link_add_form_title(context, request, form):
    """Site link add form title"""
    manager = get_parent(context, ISiteManager)
    if manager is context:
        return get_object_label(manager, request, form)
    return TITLE_SPAN_BREAK.format(
        get_object_label(manager, request, form),
        get_object_label(context, request, form))


@adapter_config(required=(ISiteContainer, IAdminLayer, SiteLinkAddForm),
                provides=IAJAXFormRenderer)
class SiteLinkAddFormRenderer(ContextRequestViewAdapter):
    """Site link add form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        target = getattr(self.view, '_v_target', self.request.context)
        return {
            'status': 'reload',
            'location': absolute_url(target, self.request, 'site-tree.html')
        }


@adapter_config(required=(ISiteLink, IAdminLayer, Interface),
                provides=ITableElementEditor)
class SiteLinkTableElementEditor(TableElementEditor):
    """Site link table element editor"""


@implementer(IPropertiesEditForm)
class SiteLinkPropertiesEditForm(AdminModalEditForm):
    """Site link properties edit form"""

    @property
    def subtitle(self):
        """Title getter"""
        translate = self.request.localizer.translate
        return translate(_("{content_name}: {label}")).format(
                content_name=translate(self.context.content_name),
                label=get_object_label(self.context, self.request, self))

    legend = _("Link properties")

    _edit_permission = MANAGE_CONTENT_PERMISSION


@adapter_config(required=(ISiteLink, IAdminLayer,
                          SiteLinkPropertiesEditForm),
                provides=IGroup)
class SiteLinkHeaderGroup(FormGroupChecker):
    """Site link header group"""

    fields = Fields(ISiteLink).select('show_header', 'navigation_header')
    checker_fieldname = 'show_header'


@adapter_config(required=(ISiteLink, IAdminLayer,
                          SiteLinkPropertiesEditForm),
                provides=IAJAXFormRenderer)
class SiteLinkPropertiesEditFormRenderer(ContextRequestViewAdapter):
    """Site link properties edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        container = get_parent(self.context, ISiteManager)
        return {
            'status': 'success',
            'message': self.request.localizer.translate(self.view.success_message),
            'callbacks': [
                get_json_table_row_refresh_callback(container, self.request,
                                                    SiteContainerTreeTable, self.context)
            ]
        }


@adapter_config(required=(ISiteLink, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentVisibility)
def site_link_dashboard_visibility(context, request, column):
    """Site link dashboard visibility"""
    if not column.has_permission(context):
        return False, ''
    icon_class = column.object_data.get('ams-icon-on' if context.visible else 'ams-icon-off')
    if not IWorkflowPublicationInfo(context.__parent__).is_published(request):
        icon_class += ' text-danger'
    hint = request.localizer.translate(_("Click to switch link visibility"))
    return True, f'<i class="fa-fw {icon_class} hint"' \
                 f'   data-original-title="{hint}"></i>'


#
# Internal link components
#

@viewlet_config(name='add-links.divider',
                context=ISiteContainer, layer=IAdminLayer, view=ISiteTreeTable,
                manager=IContextAddingsViewletManager, weight=49,
                permission=CREATE_CONTENT_PERMISSION)
class SiteLinksAddMenuDivider(MenuDivider):
    """Site links add menus divider"""


@viewlet_config(name='add-internal-link.menu',
                context=ISiteContainer, layer=IAdminLayer, view=ISiteTreeTable,
                manager=IContextAddingsViewletManager, weight=50,
                permission=CREATE_CONTENT_PERMISSION)
class InternalSiteLinkAddMenu(MenuItem):
    """Internal site link add menu"""

    label = _("Add internal link...")
    icon_class = 'fas fa-sign-in-alt fa-rotate-270'

    href = 'add-internal-link.html'
    modal_target = True


class IInternalSiteLinkAddFormFields(IInternalSiteLink):
    """Internal content link add form fields interface"""

    parent = Int(title=_("Parent"),
                 description=_("Link's parent"),
                 required=True)


@ajax_form_config(name='add-internal-link.html',
                  context=ISiteContainer, layer=IPyAMSLayer,
                  permission=CREATE_CONTENT_PERMISSION)
class InternalSiteLinkAddForm(SiteLinkAddForm):
    """Internal site link add form"""

    subtitle = _("New internal link")
    legend = _("New internal link properties")

    fields = Fields(IInternalSiteLinkAddFormFields).select('reference', 'force_canonical_url',
                                                           'navigation_title', 'parent')
    fields['parent'].widget_factory = SiteManagerFoldersSelectorFieldWidget
    content_factory = IInternalSiteLink

    def update_content(self, link, data):
        super().update_content(link, data)
        data = data.get(self, data)
        link.reference = data['reference']


@ajax_form_config(name='properties.html',
                  context=IInternalSiteLink, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class InternalSiteLinkPropertiesEditForm(SiteLinkPropertiesEditForm):
    """Internal site link properties edit form"""

    fields = Fields(IInternalSiteLink).select('reference', 'force_canonical_url',
                                              'navigation_title')


@adapter_config(required=(IInternalSiteLink, IAdminLayer, IModalEditForm),
                provides=IFormTitle)
def internal_site_link_edit_form_title(context, request, form):
    """Internal site link edit form title"""
    parent = get_parent(context, IContainer)
    return query_adapter(IFormTitle, request, parent, form)


@adapter_config(required=(IInternalSiteLink, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentLabel)
def internal_content_link_dashboard_label(context, request, column):
    """Internal content link dashboard label"""
    label = II18n(context).query_attribute('navigation_title', request=request)
    if not label:
        target = context.target
        if target is not None:
            label = get_object_label(target, request)
    translate = request.localizer.translate
    return f'{label} <i class="ml-1 fas fa-sign-in-alt fa-rotate-270 hint" ' \
           f'data-original-title="{translate(context.content_name)}"></i>'


def get_internal_link_adapter(context, request, column, interface):
    """Get internal link column adapter"""
    target = context.target
    if target is not None:
        value = request.registry.queryMultiAdapter((target, request, column), interface)
        return f'({value.strip()})' if value is not None else MISSING_INFO
    return MISSING_INFO


@adapter_config(required=(IInternalSiteLink, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentType)
def internal_content_link_type(context, request, column):
    """Internal content link dashboard type"""
    return get_internal_link_adapter(context, request, column, IDashboardContentType)


@adapter_config(required=(IInternalSiteLink, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentNumber)
def internal_content_link_dashboard_number(context, request, column):
    """Internal content link dashboard number"""
    return get_internal_link_adapter(context, request, column, IDashboardContentNumber)


@adapter_config(required=(IInternalSiteLink, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentStatus)
def internal_content_link_dashboard_status(context, request, column):
    """Internal content link dashboard status"""
    return get_internal_link_adapter(context, request, column, IDashboardContentStatus)


@adapter_config(required=(IInternalSiteLink, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentStatusDatetime)
def internal_content_link_dashboard_status_datetime(context, request, column):
    """Internal content link dashboard status datetime"""
    return get_internal_link_adapter(context, request, column, IDashboardContentStatusDatetime)


@adapter_config(required=(IInternalSiteLink, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentVersion)
def internal_content_link_dashboard_version(context, request, column):
    """Internal content link dashboard version"""
    return get_internal_link_adapter(context, request, column, IDashboardContentVersion)


@adapter_config(required=(IInternalSiteLink, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentModifier)
def internal_content_link_dashboard_modifier(context, request, column):
    """Internal content link dashboard modifier"""
    return get_internal_link_adapter(context, request, column, IDashboardContentModifier)


@adapter_config(required=(IInternalSiteLink, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentOwner)
def internal_content_link_dashboard_owner(context, request, column):
    """Internal content link dashboard owner"""
    return get_internal_link_adapter(context, request, column, IDashboardContentOwner)


#
# External site link components
#

@viewlet_config(name='add-external-link.menu',
                context=ISiteContainer, layer=IAdminLayer, view=ISiteTreeTable,
                manager=IContextAddingsViewletManager, weight=60,
                permission=CREATE_CONTENT_PERMISSION)
class ExternalSiteLinkAddMenu(MenuItem):
    """External site link add menu"""

    label = _("Add external link...")
    icon_class = 'fas fa-link'

    href = 'add-external-link.html'
    modal_target = True


class IExternalSiteLinkAddFormFields(IExternalSiteLink):
    """External site link add form fields interface"""

    parent = Int(title=_("Parent"),
                 description=_("Link's parent"),
                 required=True)


@ajax_form_config(name='add-external-link.html',
                  context=ISiteContainer, layer=IPyAMSLayer,
                  permission=CREATE_CONTENT_PERMISSION)
class ExternalSiteLinkAddForm(SiteLinkAddForm):
    """External site link add form"""

    subtitle = _("New external link")
    legend = _("New external link properties")

    fields = Fields(IExternalSiteLinkAddFormFields).select('url', 'navigation_title',
                                                           'parent')
    fields['parent'].widget_factory = SiteManagerFoldersSelectorFieldWidget
    content_factory = IExternalSiteLink

    def update_content(self, link, data):
        super().update_content(link, data)
        data = data.get(self, data)
        link.url = data['url']


@ajax_form_config(name='properties.html',
                  context=IExternalSiteLink, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ExternalSiteLinkPropertiesEditForm(SiteLinkPropertiesEditForm):
    """External site link properties edit form"""

    fields = Fields(IExternalSiteLink).select('url', 'navigation_title')


@adapter_config(required=(IExternalSiteLink, IAdminLayer, IModalEditForm),
                provides=IFormTitle)
def external_site_link_edit_form_title(context, request, form):
    """External site link edit form title"""
    parent = get_parent(context, IContainer)
    return query_adapter(IFormTitle, request, parent, form)


@adapter_config(required=(IExternalSiteLink, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentLabel)
def external_site_link_dashboard_label(context, request, column):
    """External site link dashboard label"""
    label = II18n(context).query_attribute('navigation_title', request=request)
    if not label:
        label = context.url
    translate = request.localizer.translate
    return f'{label} <i class="ml-1 fas fa-link hint" ' \
           f'data-original-title="{translate(context.content_name)}"></i>'



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

import locale

from pyams_content.reference.pictogram import IPictogramTable
from pyams_content.reference.pictogram.interfaces import IPictogramManager, \
    IPictogramManagerTarget
from pyams_form.browser.select import SelectWidget
from pyams_form.template import widget_layout_config, widget_template_config
from pyams_form.widget import FieldWidget
from pyams_i18n.interfaces import II18n
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_zmi.interfaces import IAdminLayer


@widget_layout_config(template='templates/selection-layout.pt', layer=IAdminLayer)
@widget_template_config(template='templates/selection.pt', layer=IAdminLayer)
class PictogramManagerSelectionWidget(SelectWidget):
    """Pictogram manager selection widget"""

    pictogram_table = None
    pictogram_manager = None

    def update(self):
        super().update()
        self.pictogram_table = query_utility(IPictogramTable)
        target = get_parent(self.context, IPictogramManagerTarget)
        if target is not None:
            self.pictogram_manager = IPictogramManager(target)

    @property
    def sorted_pictograms(self):
        yield from sorted(self.pictogram_table.values(),
                          key=lambda x: locale.strxfrm(
                              (II18n(x).query_attribute('title',
                                                        request=self.request) or '').lower()))

    @property
    def available_pictograms(self):
        manager = self.pictogram_manager
        if manager is not None:
            for pictogram in self.sorted_pictograms:
                if pictogram.__name__ not in (manager.selected_pictograms or ()):
                    yield pictogram

    @property
    def selected_pictograms(self):
        manager = self.pictogram_manager
        if manager is not None:
            for name in (manager.selected_pictograms or ()):
                pictogram = self.pictogram_table.get(name)
                if pictogram is not None:
                    yield pictogram


def PictogramManagerSelectionFieldWidget(field, request):  # pylint: disable=invalid-name
    """Pictogram manager selection field widget factory"""
    return FieldWidget(field, PictogramManagerSelectionWidget(request))



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.alert.type module

"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.container.ordered import OrderedContainer
from zope.location.interfaces import ISublocations
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
from zope.traversing.interfaces import ITraversable

from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.reference.pictogram import IPictogramTable
from pyams_content.shared.alert import IAlertManager, IAlertTypesManager
from pyams_content.shared.alert.interfaces import ALERT_TYPES_MANAGER_ANNOTATION_KEY, ALERT_TYPES_VOCABULARY, IAlertType
from pyams_i18n.interfaces import II18n
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.registry import query_utility
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config

__docformat__ = 'restructuredtext'


@factory_config(IAlertType)
class AlertType(Persistent, Contained):
    """Alert type persistent class"""

    visible = FieldProperty(IAlertType['visible'])
    dashboard_label = FieldProperty(IAlertType['dashboard_label'])
    pictogram = FieldProperty(IAlertType['pictogram'])
    color = FieldProperty(IAlertType['color'])

    def get_pictogram(self):
        table = query_utility(IPictogramTable)
        return table.get(self.pictogram) if table is not None else None


@adapter_config(required=IAlertType,
                provides=IViewContextPermissionChecker)
class AlertTypePermissionChecker(ContextAdapter):
    """Alert type permission checker"""

    edit_permission = MANAGE_TOOL_PERMISSION


@factory_config(IAlertTypesManager)
class AlertTypesManager(OrderedContainer):
    """Alert types manager"""

    def get_visible_items(self):
        """Visible alert types iterator"""
        yield from filter(lambda x: x.visible, self.values())


@adapter_config(required=IAlertManager,
                provides=IAlertTypesManager)
def alerts_types_manager(context):
    """Alerts types manager adapter"""
    return get_annotation_adapter(context, ALERT_TYPES_MANAGER_ANNOTATION_KEY, IAlertTypesManager,
                                  name='++types++')


@adapter_config(name='types',
                required=IAlertManager,
                provides=ITraversable)
class AlertsManagerTypesNamespace(ContextAdapter):
    """Alerts manager types ++gravity++ namespace"""

    def traverse(self, name, furtherpath=None):
        return IAlertTypesManager(self.context)


@adapter_config(name='types',
                required=IAlertManager,
                provides=ISublocations)
class AlertsManagerTypesSublocations(ContextAdapter):
    """Alerts manager types sub-locations adapter"""

    def sublocations(self):
        return IAlertTypesManager(self.context).values()


@vocabulary_config(name=ALERT_TYPES_VOCABULARY)
class AlertTypesVocabulary(SimpleVocabulary):
    """Alert types vocabulary"""

    def __init__(self, context):
        terms = []
        parent = get_parent(context, IAlertManager)
        if parent is not None:
            request = check_request()
            manager = IAlertTypesManager(parent)
            terms = [
                SimpleTerm(alert_type.__name__,
                           title=II18n(alert_type).query_attribute('label', request=request))
                for alert_type in manager.values()
            ]
        super().__init__(terms)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.site.container module

This module provides base site container mixin class
"""

import json

from pyramid.location import lineage
from zope.interface import implementer
from zope.intid import IIntIds
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.shared.site.interfaces import IBaseSiteItem, ISiteContainer, \
    ISiteElementNavigation, ISiteFolder, ISiteManager, SITE_CONTENT_VOCABULARY
from pyams_i18n.interfaces import II18n
from pyams_utils.finder import find_objects_providing
from pyams_utils.registry import get_pyramid_registry, get_utility
from pyams_utils.request import check_request, query_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config


__docformat__ = 'restructuredtext'


@implementer(ISiteContainer)
class SiteContainerMixin:
    """Site container mixin class"""

    def get_visible_items(self, request=None):

        def check_item(item):
            navigation = registry.queryMultiAdapter((item, request), ISiteElementNavigation)
            return (navigation is not None) and navigation.visible

        if request is None:
            request = check_request()
        registry = get_pyramid_registry()
        yield from filter(check_item, self.values())

    def get_folders_tree(self, selected=None, permission=None):

        request = query_request()
        intids = get_utility(IIntIds)

        def get_folder_items(parent, input):
            for folder in parent.values():
                if ISiteFolder.providedBy(folder):
                    if permission is not None:
                        can_select = request.has_permission(permission, context=folder)
                    else:
                        can_select = True
                    value = {
                        'id': intids.queryId(folder),
                        'text': II18n(folder).query_attribute('title', request=request),
                        'state': {
                            'expanded': folder in lineage(self),
                            'selected': folder is selected
                        },
                        'selectable': can_select
                    }
                    items = get_folder_items(folder, [])
                    if items:
                        value['nodes'] = items
                    input.append(value)
            return input

        # get child folders
        items = get_folder_items(self, [])

        # get parents folders
        container = self
        while ISiteContainer.providedBy(container):
            can_select = request.has_permission(permission, context=container) \
                if permission is not None else True
            items = [{
                'id': intids.queryId(container),
                'text': II18n(container).query_attribute('title', request=request),
                'state': {
                    'expanded': True,
                    'selected': container is selected
                },
                'selectable': can_select,
                'nodes': items
            }]
            container = container.__parent__

        return json.dumps(items)


@vocabulary_config(name=SITE_CONTENT_VOCABULARY)
class SiteManagerContentsVocabulary(SimpleVocabulary):
    """Site manager folders vocabulary"""

    def __init__(self, context):
        terms = []
        site = get_parent(context, ISiteManager)
        if site is not None:
            request = query_request()
            intids = get_utility(IIntIds)
            for folder, depth in find_objects_providing(site, IBaseSiteItem, with_depth=True):
                terms.append(SimpleTerm(value=intids.queryId(folder),
                                        title='{}{}'.format(
                                            '- ' * depth,
                                            II18n(folder).query_attribute('title',
                                                                          request=request))))
        super(SiteManagerContentsVocabulary, self).__init__(terms)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.skin.provider.vimeo module

This module defines Vimeo external video renderer.
"""

from pyams_content.component.video.provider.interfaces import IVimeoVideoSettings
from pyams_content.component.video.skin import IExternalVideoRenderer
from pyams_content.component.video.skin.provider import BaseExternalVideoRenderer
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config

__docformat__ = 'restructuredtext'


VIMEO_PARAMS = (
    ('show_title', 'title', int),
    ('show_signature', 'byline', int),
    ('color', 'color', str),
    ('autoplay', 'autoplay', int),
    ('loop', 'loop', int)
)


@adapter_config(required=(IVimeoVideoSettings, IPyAMSLayer),
                provides=IExternalVideoRenderer)
@template_config(template='templates/vimeo-render.pt', layer=IPyAMSLayer)
class VimeoVideoRenderer(BaseExternalVideoRenderer):
    """Vimeo video renderer"""

    params = VIMEO_PARAMS



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.association.zmi.interfaces module

This module defines associations-related management interfaces.
"""

from zope.interface import Interface

from pyams_zmi.interfaces.form import IPropertiesEditForm


__docformat__ = 'restructuredtext'


class IAssociationsView(Interface):
    """Associations view marker interface"""


class IAssociationsTable(Interface):
    """Associations table marker interface"""


class IAssociationsContainerEditForm(Interface):
    """Associations container edit form marker interface"""


class IAssociationItemAddForm(Interface):
    """Association item add form marker interface"""


class IAssociationItemEditForm(IPropertiesEditForm):
    """Association item edit form marker interface"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.root.zmi.sites module

This module provides site's root view of sites, hubs and blogs.
"""

from pyramid.decorator import reify
from pyramid.interfaces import IView
from pyramid.view import view_config
from zope.container.interfaces import IContainer
from zope.interface import implementer

from pyams_content.interfaces import MANAGE_SITE_TREE_PERMISSION
from pyams_content.shared.common import IBaseSharedTool
from pyams_content.shared.common.interfaces import IDeletableElement
from pyams_content.zmi.dashboard import DashboardLabelColumn
from pyams_content.zmi.interfaces import IDashboardTable
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_workflow.interfaces import IWorkflowPublicationInfo
from pyams_zmi.helper.container import delete_container_element
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import ISiteManagementMenu
from pyams_zmi.table import IconColumn, Table, TableAdminView, TrashColumn
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


@implementer(IDashboardTable, IView)
class SiteRootSitesTable(Table):
    """Site root sub-sites table"""

    @reify
    def data_attributes(self):
        attributes = super().data_attributes
        attributes['table'].update({
            'data-ams-order': '1,asc'
        })
        return attributes

    display_if_empty = True


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteRootSitesTable),
                provides=IValues)
class SiteRootSitesTableValues(ContextRequestViewAdapter):
    """Site root sites table values"""

    @property
    def values(self):
        """Site root sites tables values getter"""
        yield from filter(IBaseSharedTool.providedBy,
                          IContainer(self.context).values())


@adapter_config(name='visible',
                required=(ISiteRoot, IAdminLayer, SiteRootSitesTable),
                provides=IColumn)
class SiteRootSitesTableVisibleColumn(IconColumn):
    """Site root sites table visible column"""

    weight = 1

    def get_icon_class(self, item):
        info = IWorkflowPublicationInfo(item, None)
        if info is None:
            return None
        if info.is_published():
            return 'fas fa-eye'
        return 'fas fa-eye-slash text-danger opaque'

    def get_icon_hint(self, item):
        info = IWorkflowPublicationInfo(item, None)
        if info is None:
            return None
        translate = self.request.localizer.translate
        if info.is_published():
            return translate(_("Visible site"))
        return translate(_("Not visible site"))


@adapter_config(name='name',
                required=(ISiteRoot, IAdminLayer, SiteRootSitesTable),
                provides=IColumn)
class SiteRootSitesNameColumn(DashboardLabelColumn):
    """Site root site name column"""

    css_classes = {
        'td': 'text-truncate'
    }


@adapter_config(name='trash',
                required=(ISiteRoot, IAdminLayer, SiteRootSitesTable),
                provides=IColumn)
class SiteRootSitesTrashColumn(TrashColumn):
    """Site root sites trash column"""

    hint = _("Delete shared site")
    permission = MANAGE_SITE_TREE_PERMISSION

    def checker(self, item):
        """Trash column checker"""
        deletable = IDeletableElement(item, None)
        if (deletable is not None) and not deletable.is_deletable():
            return False
        return super().has_permission(item)


@viewlet_config(name='root-sites.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=ISiteManagementMenu, weight=5,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootSitesMenu(NavigationMenuItem):
    """Site root sites menu"""

    label = _("Site tree")
    icon_class = 'fas fa-sitemap'
    href = '#root-sites.html'


@pagelet_config(name='root-sites.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootSitesView(TableAdminView):
    """Site root sites view"""

    title = _("Sites, hubs and blogs")

    table_class = SiteRootSitesTable
    table_label = _("Site content")


@view_config(name='delete-element.json',
             context=ISiteRoot, request_type=IPyAMSLayer,
             permission=MANAGE_SITE_TREE_PERMISSION,
             renderer='json', xhr=True)
def delete_siteroot_element(request):
    """Site root delete view"""
    return delete_container_element(request)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.links.html module

This module defines components which are used to handle associations in HTML
components.
"""

__docformat__ = 'restructuredtext'

import re
from pyquery import PyQuery
from zope.lifecycleevent import ObjectCreatedEvent

from pyams_content.component.association import IAssociationContainer
from pyams_content.component.extfile import IBaseExtFile
from pyams_content.component.links import IExternalLink, IInternalLink, IMailtoLink
from pyams_i18n.interfaces import II18n
from pyams_sequence.interfaces import ISequentialIntIds
from pyams_utils.factory import get_object_factory
from pyams_utils.registry import get_pyramid_registry, get_utility
from pyams_utils.request import check_request
from pyams_utils.url import absolute_url


FULL_EMAIL = re.compile(r'(.*) <(.*)>')


def check_content_links(context, body, lang, notify=True):  # pylint: disable=too-many-branches, too-many-statements, too-many-locals
    """Check for link associations from HTML content"""
    associations = IAssociationContainer(context, None)
    if associations is None:
        return
    registry = get_pyramid_registry()
    html = PyQuery('<html>{0}</html>'.format(body))
    for link in html('a[href]'):
        link_info = None
        has_link = False
        href = link.attrib['href']
        if href.startswith('oid://'):
            sequence = get_utility(ISequentialIntIds)
            oid = sequence.get_full_oid(href.split('//', 1)[1])
            for association in associations.values():
                internal_info = IInternalLink(association, None)
                if (internal_info is not None) and (internal_info.reference == oid):
                    has_link = True
                    break
            if not has_link:
                factory = get_object_factory(IInternalLink)
                if factory is not None:
                    link_info = factory()
                    link_info.visible = False
                    link_info.reference = oid
                    link_info.title = {lang: link.attrib.get('title') or link.text}
        elif href.startswith('mailto:'):
            name = None
            email = href[7:]
            if '<' in email:
                groups = FULL_EMAIL.findall(email)
                if groups:
                    name, email = groups[0]
            for association in associations.values():
                mailto_info = IMailtoLink(association, None)
                if (mailto_info is not None) and (mailto_info.address == email):
                    has_link = True
                    break
            if not has_link:
                factory = get_object_factory(IMailtoLink)
                if factory is not None:
                    link_info = factory()
                    link_info.visible = False
                    link_info.address = email
                    link_info.address_name = name or email
                    link_info.title = {lang: link.attrib.get('title') or link.text}
        elif href.startswith('http://') or href.startswith('https://'):
            for association in associations.values():
                external_info = IExternalLink(association, None)
                if (external_info is not None) and (external_info.url == href):
                    has_link = True
                    break
                else:
                    extfile_info = IBaseExtFile(association, None)
                    if extfile_info is not None:
                        request = check_request()
                        extfile_url = absolute_url(
                            II18n(extfile_info).query_attribute('data', request=request),
                            request=request)
                        if extfile_url.endswith(href):
                            has_link = True
                            break
            if not has_link:
                factory = get_object_factory(IExternalLink)
                if factory is not None:
                    link_info = factory()
                    link_info.visible = False
                    link_info.url = href
                    link_info.title = {lang: link.attrib.get('title') or link.text}
        if link_info is not None:
            registry.notify(ObjectCreatedEvent(link_info))
            associations.append(link_info, notify=notify)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.zmi.interfaces module

"""

__docformat__ = 'restructuredtext'

from zope.interface import Interface

from pyams_viewlet.interfaces import IViewletManager


class IParagraphContainerBaseTable(Interface):
    """Paragraph container base table marker interface"""


class IParagraphContainerFullTable(IParagraphContainerBaseTable):
    """Paragraph container table marker interface"""


class IParagraphContainerView(Interface):
    """Paragraph container view marker interface"""


class IParagraphAddForm(Interface):
    """Paragraph add form marker interface"""


class IParagraphEditForm(Interface):
    """Paragraph edit form marker interface"""


class IInnerParagraphEditForm(Interface):
    """Inner paragraph edit form marker interface"""


class IParagraphRendererSettingsEditForm(Interface):
    """Paragraph renderer settings edit form interface"""


class IParagraphTitleToolbar(IViewletManager):
    """Paragraph title toolbar viewlet manager interface"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.provider.vimeo module

This module defines components which are used for Vimeo videos integration.
"""

import re

from persistent import Persistent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.video import IExternalVideo, IExternalVideoSettings, external_video_settings
from pyams_content.component.video.interfaces import IExternalVideoProvider
from pyams_content.component.video.provider.interfaces import IVimeoVideoSettings
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.registry import utility_config

__docformat__ = 'restructuredtext'

from pyams_content import _


VIMEO_BASE_URL = re.compile('https://vimeo.com/([0-9]+)')


@factory_config(IVimeoVideoSettings)
class VimeoVideoSettings(Persistent):
    """Vimeo video settings"""

    _video_id = FieldProperty(IVimeoVideoSettings['video_id'])
    show_title = FieldProperty(IVimeoVideoSettings['show_title'])
    show_signature = FieldProperty(IVimeoVideoSettings['show_signature'])
    color = FieldProperty(IVimeoVideoSettings['color'])
    autoplay = FieldProperty(IVimeoVideoSettings['autoplay'])
    loop = FieldProperty(IVimeoVideoSettings['loop'])
    allow_fullscreen = FieldProperty(IVimeoVideoSettings['allow_fullscreen'])
    width = FieldProperty(IVimeoVideoSettings['width'])
    height = FieldProperty(IVimeoVideoSettings['height'])

    @property
    def video_id(self):
        return self._video_id

    @video_id.setter
    def video_id(self, value):
        if value:
            match = VIMEO_BASE_URL.match(value)
            if match:
                value = match.groups()[0]
        self._video_id = value


@utility_config(name='vimeo',
                provides=IExternalVideoProvider)
class VimeoVideoProvider:
    """Vimeo video provider"""

    label = _("Vimeo")
    weight = 30

    settings_interface = IVimeoVideoSettings


@adapter_config(required=IExternalVideo,
                provides=IVimeoVideoSettings)
def vimeo_video_settings(context):
    """Vimeo video settings factory"""
    return external_video_settings(context, provider_name='vimeo')


@adapter_config(required=VimeoVideoProvider,
                provides=IExternalVideoSettings)
def vimeo_video_provider_settings_factory(context):
    """Vimeo video provider settings factory"""
    return IVimeoVideoSettings



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_content.feature.alert import IAlertManagerInfo
from pyams_content.interfaces import MANAGE_SITE_ROOT_PERMISSION
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormContent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='alerts-manager.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=760,
                permission=MANAGE_SITE_ROOT_PERMISSION)
class AlertsManagerMenu(NavigationMenuItem):
    """Alerts manager menu"""

    label = _("Alerts management")
    href = '#alerts-manager.html'


@ajax_form_config(name='alerts-manager.html',
                  context=ISiteRoot, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_ROOT_PERMISSION)
class AlertsManagerEditForm(AdminEditForm):
    """Alerts manager edit form"""

    title = _("Alerts management")
    legend = _("Alerts management views")

    fields = Fields(IAlertManagerInfo)


@adapter_config(required=(ISiteRoot, IPyAMSLayer, AlertsManagerEditForm),
                provides=IFormContent)
def site_root_alert_edit_form_content(context, request, form):
    """Site root alerts edit form content getter"""
    return IAlertManagerInfo(context)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.site.interfaces module

"""

from collections import OrderedDict

from zope.annotation import IAttributeAnnotatable
from zope.container.constraints import containers, contains
from zope.container.interfaces import IContainer
from zope.interface import Attribute, Interface
from zope.location.interfaces import IContained
from zope.schema import Bool, Choice, Text, URI
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.interfaces import IBaseContent
from pyams_content.shared.common.interfaces import IBaseSharedTool, IDeletableElement, ISharedSite
from pyams_content.shared.common.interfaces.types import DATA_TYPES_VOCABULARY
from pyams_content.shared.topic import ITopic, IWfTopic
from pyams_i18n.schema import I18nTextField, I18nTextLineField
from pyams_sequence.interfaces import IInternalReference, IInternalReferencesList, \
    ISequentialIdTarget
from pyams_workflow.interfaces import IWorkflowPublicationSupport


__docformat__ = 'restructuredtext'

from pyams_content import _


SITE_CONTAINER_REDIRECT_MODE = 'redirect'
SITE_CONTAINER_TEMPLATE_MODE = 'template'

SITE_CONTAINER_NAVIGATION_MODES = OrderedDict((
    (SITE_CONTAINER_REDIRECT_MODE, _("Redirect to first visible sub-folder or content")),
    (SITE_CONTAINER_TEMPLATE_MODE, _("Use presentation template"))
))

SITE_CONTAINER_NAVIGATION_MODES_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v, title=t)
    for v, t in SITE_CONTAINER_NAVIGATION_MODES.items()
])

SITE_CONTENT_VOCABULARY = 'pyams_content.site.content'


class ISiteElement(IContained, IDeletableElement):
    """Base site element interface"""

    containers('.ISiteContainer')

    content_name = Attribute("Content name")


class ISiteElementNavigation(Interface):
    """Site element navigation interface"""

    visible = Attribute("Visible element?")


class IBaseSiteItem(IContained, IWorkflowPublicationSupport):
    """Base site item class"""


class ISiteContainer(IContainer, IBaseSiteItem):
    """Base site container interface"""

    contains(ISiteElement)

    navigation_mode = Choice(title=_("Navigation mode"),
                             description=_("Site behaviour when navigating to site URL"),
                             required=True,
                             vocabulary=SITE_CONTAINER_NAVIGATION_MODES_VOCABULARY,
                             default=SITE_CONTAINER_TEMPLATE_MODE)

    shared_content_type = Attribute("Shared content type")

    shared_content_factory = Attribute("Shared content factory")

    def get_visible_items(self, request=None):
        """Iterator over container visible items"""

    def get_folders_tree(self, selected=None):
        """Get site tree in JSON format"""


#
# Site folder interface
#

SITE_FOLDERS_VOCABULARY = 'pyams_content.site.folders'


class ISiteFolder(IBaseContent, ISiteElement, ISiteContainer, ISequentialIdTarget):
    """Site folder interface

    A site folder is made to contain sub-folders and topics
    """

    header = I18nTextField(title=_("Header"),
                           description=_("Heading displayed according to presentation template"),
                           required=False)

    description = I18nTextField(title=_("Meta-description"),
                                description=_("The folder's description is 'hidden' into HTML's "
                                              "page headers; but it can be seen, for example, "
                                              "in some search engines results as content's "
                                              "description; if description is empty, content's "
                                              "header will be used."),
                                required=False)

    notepad = Text(title=_("Notepad"),
                   description=_("Internal information to be known about this content"),
                   required=False)

    visible_in_list = Bool(title=_("Visible in folders list"),
                           description=_("If 'no', folder will not be displayed into folders "
                                         "list"),
                           required=True,
                           default=True)

    navigation_title = I18nTextLineField(title=_("Navigation title"),
                                         description=_("Folder's title displayed in navigation "
                                                       "pages; original title will be used if "
                                                       "none is specified"),
                                         required=False)

    navigation_mode = Choice(title=_("Navigation mode"),
                             description=_("Folder behaviour when navigating to folder URL"),
                             required=True,
                             vocabulary=SITE_CONTAINER_NAVIGATION_MODES_VOCABULARY,
                             default=SITE_CONTAINER_REDIRECT_MODE)


#
# Site manager interface
#

PYAMS_SITES_VOCABULARY = 'pyams_content.sites'


class ISiteManager(ISharedSite, ISiteContainer, IBaseSharedTool,
                   IDeletableElement, ISequentialIdTarget):
    """Site manager interface"""

    contains(ISiteElement)

    folder_factory = Attribute("Folder factory")

    header = I18nTextField(title=_("Header"),
                           description=_("Site's header is generally displayed in page header"),
                           required=False)

    description = I18nTextField(title=_("Meta-description"),
                                description=_("The site's description is 'hidden' into HTML's "
                                              "page headers; but it can be seen, for example, "
                                              "in some search engines results as content's "
                                              "description; if description is empty, content's "
                                              "header will be used."),
                                required=False)

    notepad = Text(title=_("Notepad"),
                   description=_("Internal information to be known about this content"),
                   required=False)


#
# Site topics interfaces
#

SITE_TOPIC_CONTENT_TYPE = 'site-topic'
SITE_TOPIC_CONTENT_NAME = _("Site topic")


class IWfSiteTopic(IWfTopic, IInternalReferencesList):
    """Site topic interface"""

    data_type = Choice(title=_("Data type"),
                       description=_("Type of content data"),
                       required=False,
                       vocabulary=DATA_TYPES_VOCABULARY)


class ISiteTopic(ITopic, ISiteElement):
    """Workflow managed site topic interface"""


#
# Site links interfaces
#

class ISiteLink(ISiteElement):
    """Content link interface"""
    
    navigation_title = I18nTextLineField(title=_("Navigation title"),
                                         description=_("Alternate content's title displayed in "
                                                       "navigation pages; original title will be "
                                                       "used if none is specified"),
                                         required=False)

    show_header = Bool(title=_("Show header?"),
                       description=_("If 'no', no header will be displayed"),
                       required=False,
                       default=True)

    navigation_header = I18nTextField(title=_("Navigation header"),
                                      description=_("Alternate content's header displayed in "
                                                    "navigation pages; original header will be "
                                                    "used if none is specified"),
                                      required=False)

    visible = Bool(title=_("Visible?"),
                   description=_("If 'no', link is not visible"),
                   required=True,
                   default=True)


class IInternalSiteLink(ISiteLink, IInternalReference):
    """Internal content site link interface"""

    force_canonical_url = Bool(title=_("Force canonical URL?"),
                               description=_("If 'yes', link to internal references will "
                                             "use their canonical instead of relative URL"),
                               required=True,
                               default=False)


class IExternalSiteLink(ISiteLink, IAttributeAnnotatable):
    """External site link interface"""

    url = URI(title=_("Target URL"),
              description=_("URL used to access external resource"),
              required=True)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.review.interfaces module

"""

from zope.annotation import IAttributeAnnotatable
from zope.container.constraints import containers, contains
from zope.interface import Attribute, Interface, implementer
from zope.interface.interfaces import IObjectEvent, ObjectEvent
from zope.location.interfaces import IContained
from zope.schema import Bool, Choice, Datetime, Text, TextLine
from zope.schema.interfaces import IContainer

from pyams_security.schema import PrincipalField, PrincipalsSetField


__docformat__ = 'restructuredtext'

from pyams_content import _


COMMENT_TYPES = {
    'request': _("Review request"),
    'comment': _("Reviewer comment")
}


class ICommentAddedEvent(IObjectEvent):
    """Comment added event interface"""

    comment = Attribute("New comment")


@implementer(ICommentAddedEvent)
class CommentAddedEvent(ObjectEvent):
    """Comment added event"""

    def __init__(self, object, comment):
        super().__init__(object)
        self.comment = comment


class IReviewComment(IContained, IAttributeAnnotatable):
    """Review comment interface"""

    containers('.IReviewComments')

    owner = PrincipalField(title=_("Comment writer"),
                           required=True)

    reviewers = TextLine(title=_("Content reviewers"),
                         required=False)

    comment_type = Choice(title=_("Comment type"),
                          values=COMMENT_TYPES.keys(),
                          required=True,
                          default='comment')

    comment = Text(title=_("Comment body"),
                   required=True)

    is_reviewer_comment = Bool(title=_("Reviewer comment?"),
                               required=True,
                               default=False)

    creation_date = Datetime(title=_("Creation date"),
                             required=False)


REVIEW_COMMENTS_ANNOTATION_KEY = 'pyams_content.review.comments'


class IReviewComments(IContainer):
    """Review comment container interface"""

    contains(IReviewComment)

    reviewers = PrincipalsSetField(title=_("Reviewers list"),
                                   description=_("List of principals which reviewed the comment"),
                                   required=False)

    def clear(self):
        """Remove all comments"""

    def add_comment(self, comment):
        """Add given comment to list"""


class IReviewManager(Interface):
    """Content review interface"""

    def ask_review(self, reviewers, comment, notify=True, request=None):
        """Ask for content review"""


class IReviewTarget(Interface):
    """Review target marker interface

    This interface is used to mark contents which can handle review.
    """



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.interfaces module

This module defines custom interfaces related to management interface
of shared contents.
"""

__docformat__ = 'restructuredtext'

from zope.interface import Interface

from pyams_form.interfaces.form import IAddForm


class ISharedContentAddForm(IAddForm):
    """Shared content add form marker interface"""


class ISharedContentPropertiesMenu(Interface):
    """Shared content properties menu marker interface"""


class IContributorRestrictionsEditForm(Interface):
    """Contributor restrictions edit form marker interface"""


class IManagerRestrictionsEditForm(Interface):
    """Manager restrictions edit form marker interface"""


class IManagerRestrictionsGroup(Interface):
    """Manager restrictions workflow group marker interface"""


class IWorkflowDeleteFormTarget(Interface):
    """Interface used to get target of a workflow delete form"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.navigation.zmi.container module

"""

from zope.interface import implementer

from pyams_content.component.association.zmi.container import AssociationsTable
from pyams_content.feature.navigation import IMenusContainer
from pyams_content.feature.navigation.interfaces import IMenusContainerTarget
from pyams_content.feature.navigation.zmi.interfaces import IMenusContainerEditForm, IMenusTable
from pyams_form.ajax import ajax_form_config
from pyams_form.interfaces.form import IInnerSubForm
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.viewlet import IContentSuffixViewletManager
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, NullAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalDisplayForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.form import IPropertiesEditForm
from pyams_zmi.table import IconColumn, InnerTableAdminView, TableGroupSwitcher

__docformat__ = 'restructuredtext'

from pyams_content import _


@ajax_form_config(name='menus-modal.html',
                  context=IMenusContainerTarget, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
@implementer(IMenusContainerEditForm)
class MenusModalEditForm(AdminModalDisplayForm):
    """Menus modal edit form"""

    modal_class = 'modal-xl'
    subtitle = _("Navigation menus")


@factory_config(IMenusTable)
class MenusTable(AssociationsTable):
    """Menus table"""

    container_class = IMenusContainer


@adapter_config(required=(IMenusContainerTarget, IAdminLayer, IMenusTable),
                provides=IValues)
class MenusTableValues(ContextRequestViewAdapter):
    """Menus table values"""

    @property
    def values(self):
        """Menus container values getter"""
        yield from IMenusContainer(self.context).values()


@adapter_config(name='icon',
                required=(IMenusContainer, IAdminLayer, IMenusTable),
                provides=IColumn)
class MenusIconColumn(NullAdapter):
    """Menus table icon column"""


@adapter_config(name='size',
                required=(IMenusContainer, IAdminLayer, IMenusTable),
                provides=IColumn)
class MenusSizeColumn(NullAdapter):
    """Menus table size column"""


@adapter_config(name='dynamic',
                required=(IMenusContainer, IAdminLayer, IMenusTable),
                provides=IColumn)
class MenusDynamicColumn(IconColumn):
    """Menus table dynamic column"""

    icon_class = 'fab fa-elementor'
    hint = _("Dynamic menu")

    weight = 45

    def get_icon_class(self, item):
        """Icon class getter"""
        return self.icon_class if item.dynamic_menu else None


#
# Main menus table
#

@viewlet_config(name='associations-table',
                context=IMenusContainerTarget, layer=IAdminLayer,
                view=IMenusContainerEditForm,
                manager=IContentSuffixViewletManager, weight=20)
class MenusTableView(InnerTableAdminView):
    """Menus table view"""

    table_class = IMenusTable
    table_label = _("Navigation menus")

    container_intf = IMenusContainer


@adapter_config(name='associations-group',
                required=(IMenusContainerTarget, IAdminLayer, IPropertiesEditForm),
                provides=IInnerSubForm, force_implements=False)
class MenusGroup(TableGroupSwitcher):
    """Menus table group"""

    legend = _("Navigation menus")

    table_class = IMenusTable
    container_intf = IMenusContainer

    weight = 30



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_content.feature.filter import IFilter
from pyams_content.feature.filter.interfaces import IAggregatedPortletRendererSettings, \
    IFilterProcessorAggregationsHandler, IFiltersContainer
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_portal.interfaces import IPortletRenderer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.dict import DotDict
from pyams_viewlet.viewlet import BaseContentProvider, contentprovider_config

__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(IFilter, IPyAMSUserLayer, IAggregatedPortletRendererSettings),
                provides=IFilterProcessorAggregationsHandler)
class FilterProcessorAggregationsHandler:
    """"""

    def __init__(self, filter, request, renderer_settings):
        self.filter = filter
        self.request = request
        self.renderer_settings = renderer_settings

    def get_aggregations(self, aggr, filter_type):
        filter = self.filter
        request = self.request
        current_filters = [
            value
            for param in request.params.getall(filter_type)
            for value in param.split(',')
        ]
        initial_options = self.get_displayed_options(aggr, filter.displayed_entries)
        additional_options = [
            option
            for index, option in enumerate(aggr)
            if index >= filter.displayed_entries
        ]
        is_expanded_condition = any(
            item['key'] in current_filters
            for item in additional_options
        )
        show_more = len(aggr) > filter.displayed_entries
        is_expanded = show_more and is_expanded_condition
        translate = request.localizer.translate
        placeholder = II18n(filter).query_attribute("select_placeholder")
        if not placeholder:
            placeholder = translate(_("-- No selected option --"))
        return DotDict({
            'filter': filter,
            "filter_type": filter_type,
            'current_filters': current_filters,
            'initial_options': initial_options,
            'additional_options': additional_options,
            'all_options': aggr,
            'is_active': bool(current_filters),
            'show_more': show_more,
            "select_placeholder": placeholder,
            "is_expanded": is_expanded,
            "show_more_text": translate(_("Show more")),
            "show_less_text": translate(_("Show less"))
        })

    @staticmethod
    def get_displayed_options(options, num_entries):
        return options[:num_entries]


@contentprovider_config(name='filters_default',
                        layer=IPyAMSUserLayer, view=IPortletRenderer)
@template_config(template='templates/filters-default.pt',
                 layer=IPyAMSUserLayer)
class DefaultFiltersRenderer(BaseContentProvider):
    """Default filters renderer"""

    filters = None

    def update(self, renderer_settings=None, aggregations=None):
        super().update()
        container = IFiltersContainer(renderer_settings, None)
        if container is not None:
            self.filters = container.get_processed_filters(self.context, self.request, aggregations)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.seo.interfaces module

This module provides interfaces related to search engines optimization (SEO).
"""

from zope.interface import Interface
from zope.schema import Bool

__docformat__ = 'restructuredtext'

from pyams_content import _


SEO_INFO_ANNOTATION_KEY = 'pyams_content.seo'


class ISEOContentInfo(Interface):
    """SEO content information interface"""

    include_sitemap = Bool(title=_("Include sitemap"),
                           description=_("If unchecked, this content will be excluded from "
                                         "sitemap"),
                           required=True,
                           default=True)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface

from pyams_content.feature.filter.interfaces import IFilterIndexInfo, IFilterValues
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.registry import get_pyramid_registry

__docformat__ = 'restructuredtext'


@adapter_config(required=Interface,
                provides=IFilterIndexInfo)
class FilterInfo(ContextAdapter):
    """Filter index info adapter"""

    @property
    def facets(self):
        result = []
        registry = get_pyramid_registry()
        for name, values in registry.getAdapters((self.context,), IFilterValues):
            result.extend(list(values))
        return result



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.interfaces.settings module

This module defines interfaces used to handle views settings.
"""

from collections import OrderedDict

from zope.interface import Attribute, Interface
from zope.schema import Bool, Choice, Int
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
from pyams_sequence.interfaces import IInternalReferencesList
from pyams_sequence.schema import InternalReferencesListField
from pyams_thesaurus.schema import ThesaurusTermsListField

__docformat__ = 'restructuredtext'

from pyams_content import _


class IViewSettings(Interface):
    """Base interface for view settings adapters"""

    is_using_context = Attribute("Check if view settings are using context")


#
# References settings
#

VIEW_REFERENCES_SETTINGS_KEY = 'pyams_content.view.references'

ALWAYS_REFERENCE_MODE = 'always'
IFEMPTY_REFERENCE_MODE = 'if_empty'
ONLY_REFERENCE_MODE = 'only'

REFERENCES_MODES = OrderedDict((
    (ALWAYS_REFERENCE_MODE, _("Always include selected internal references")),
    (IFEMPTY_REFERENCE_MODE, _("Include selected internal references only if view is empty")),
    (ONLY_REFERENCE_MODE, _("Include ONLY selected references (no search will be made)"))
))

REFERENCES_MODES_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v, title=t)
    for v, t in REFERENCES_MODES.items()
])


class IViewInternalReferencesSettings(IViewSettings, IInternalReferencesList):
    """View internal references settings"""

    select_context_references = Bool(title=_("Select context references?"),
                                     description=_("If 'non', references imposed by the context "
                                                   "will not be used"),
                                     required=False,
                                     default=True)

    references = InternalReferencesListField(title=_("Other references"),
                                             description=_("List of internal references"),
                                             required=False)

    references_mode = Choice(title=_("Internal references usage"),
                             description=_("Specify how selected references are included into "
                                           "view results"),
                             vocabulary=REFERENCES_MODES_VOCABULARY,
                             required=True,
                             default=ALWAYS_REFERENCE_MODE)

    exclude_context = Bool(title=_("Exclude context?"),
                           description=_("If 'yes', context will be excluded from results list"),
                           required=True,
                           default=True)

    def get_references(self, context):
        """Get all references for given context"""


#
# Tags settings
#

VIEW_TAGS_SETTINGS_KEY = 'pyams_content.view.tags'


class IViewTagsSettings(IViewSettings):
    """View tags settings"""

    select_context_tags = Bool(title=_("Select context tags?"),
                               description=_("If 'yes', tags will be extracted from context"),
                               required=False,
                               default=False)

    tags = ThesaurusTermsListField(title=_("Other tags"),
                                   required=False)

    def get_tags(self, context):
        """Get all tags for given context"""

    def get_tags_index(self, context):
        """Get all tags index values for given context"""


#
# Themes settings
#

VIEW_THEMES_SETTINGS_KEY = 'pyams_content.view.themes'


class IViewThemesSettings(IViewSettings):
    """View themes settings"""

    select_context_themes = Bool(title=_("Select context themes?"),
                                 description=_("If 'yes', themes will be extracted from context"),
                                 required=False,
                                 default=False)

    themes = ThesaurusTermsListField(title=_("Other themes"),
                                     required=False)

    include_subthemes = Bool(title=_("Include all subthemes?"),
                             description=_("If 'yes', subthemes of selected themes will also "
                                           "be used to search contents"),
                             required=False,
                             default=False)

    def get_themes(self, context):
        """Get all themes for given context"""

    def get_themes_index(self, context):
        """Get all themes index values for given context"""


#
# Collections settings
#

VIEW_COLLECTIONS_SETTINGS_KEY = 'pyams_content.view.collections'


class IViewCollectionsSettings(IViewSettings):
    """View collections settings"""

    select_context_collections = Bool(title=_("Select context collections?"),
                                      description=_("If 'yes', collections will be extracted "
                                                    "from context"),
                                      required=False,
                                      default=False)

    collections = ThesaurusTermsListField(title=_("Other collections"),
                                          required=False)

    def get_collections(self, context):
        """Get all collections for given context"""

    def get_collections_index(self, context):
        """Get all collections index values for given context"""



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.zmi.html module

This module provides management interface components for raw source code and rich text
paragraphs.
"""

from pyramid.events import subscriber
from zope.interface import alsoProvides

from pyams_content.component.paragraph import IParagraphContainer, IParagraphContainerTarget
from pyams_content.component.paragraph.interfaces.html import HTML_PARAGRAPH_ICON_CLASS, \
    HTML_PARAGRAPH_NAME, HTML_PARAGRAPH_TYPE, IHTMLParagraph, IRawParagraph, \
    RAW_PARAGRAPH_ICON_CLASS, RAW_PARAGRAPH_NAME, RAW_PARAGRAPH_TYPE
from pyams_content.component.paragraph.zmi import BaseParagraphAddForm, BaseParagraphAddMenu
from pyams_content.component.paragraph.zmi.interfaces import IParagraphContainerBaseTable
from pyams_form.ajax import ajax_form_config
from pyams_form.interfaces.form import IFormUpdatedEvent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import DEFAULT_RENDERER_NAME, HIDDEN_RENDERER_NAME
from pyams_utils.interfaces.data import IObjectData
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.form import IPropertiesEditForm
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager

__docformat__ = 'restructuredtext'


#
# Raw paragraphs forms
#

@viewlet_config(name='add-raw-paragraph.menu',
                context=IParagraphContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerBaseTable,
                manager=IContextAddingsViewletManager, weight=900)
class RawParagraphAddMenu(BaseParagraphAddMenu):
    """Raw paragraph add menu"""

    label = RAW_PARAGRAPH_NAME
    icon_class = RAW_PARAGRAPH_ICON_CLASS

    factory_name = RAW_PARAGRAPH_TYPE
    href = 'add-raw-paragraph.html'


@ajax_form_config(name='add-raw-paragraph.html',
                  context=IParagraphContainer, layer=IPyAMSLayer)
class RawParagraphAddForm(BaseParagraphAddForm):
    """Raw paragraph add form"""

    content_factory = IRawParagraph


@subscriber(IFormUpdatedEvent,
            form_selector=RawParagraphAddForm)
@subscriber(IFormUpdatedEvent,
            form_selector=IPropertiesEditForm,
            context_selector=IRawParagraph)
def handle_raw_paragraph_form_update(event):
    """Handle raw paragraph form update"""
    body = event.form.widgets.get('body')
    if body is not None:
        body.add_widgets_class('height-100')
        body.widget_css_class = 'editor height-300px'
        filename = 'body.html'
        if IPropertiesEditForm.providedBy(event.form):
            context = event.form.context
            if context.renderer not in (HIDDEN_RENDERER_NAME, DEFAULT_RENDERER_NAME):
                renderer = context.get_renderer()
                if renderer is not None:
                    filename = renderer.editor_filename
        for widget in body.widgets.values():
            widget.object_data = {
                'ams-filename': filename
            }
            alsoProvides(widget, IObjectData)


#
# HTML paragraphs forms
#

@viewlet_config(name='add-html-paragraph.menu',
                context=IParagraphContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerBaseTable,
                manager=IContextAddingsViewletManager, weight=50)
class HTMLParagraphAddMenu(BaseParagraphAddMenu):
    """HTML paragraph add menu"""

    label = HTML_PARAGRAPH_NAME
    icon_class = HTML_PARAGRAPH_ICON_CLASS

    factory_name = HTML_PARAGRAPH_TYPE
    href = 'add-html-paragraph.html'


@ajax_form_config(name='add-html-paragraph.html',
                  context=IParagraphContainer, layer=IPyAMSLayer)
class HTMLParagraphAddForm(BaseParagraphAddForm):
    """HTML rich text paragraph add form"""

    content_factory = IHTMLParagraph



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.site.zmi.search module

This module defines custom components which are used to handle searching
inside a site manager.
"""

from pyams_content.shared.common.zmi.dashboard import SharedToolDashboardView
from pyams_content.shared.common.zmi.search import SharedToolAdvancedSearchResultsView, \
    SharedToolQuickSearchView
from pyams_content.shared.site.interfaces import ISiteFolder, ISiteManager
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IHeaderViewletManager
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import EmptyViewlet, viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import IInnerTable


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(name='quick-search',
                required=(ISiteManager, IAdminLayer, SharedToolDashboardView),
                provides=IInnerTable)
class SiteManagerQuickSearchView(SharedToolQuickSearchView):
    """Site manager quick search view"""

    @property
    def legend(self):
        """Legend getter"""
        translate = self.request.localizer.translate
        return translate(_("Between all contents of « {} » site manager")) \
            .format(II18n(self.context).query_attribute('title', request=self.request))


@adapter_config(name='quick-search',
                required=(ISiteFolder, IAdminLayer, SharedToolDashboardView),
                provides=IInnerTable)
class SiteFolderQuickSearchView(SharedToolQuickSearchView):
    """Site folder quick search view"""

    @property
    def legend(self):
        """Legend getter"""
        translate = self.request.localizer.translate
        return translate(_("Between all contents of « {} » site folder")) \
            .format(II18n(self.context).query_attribute('title', request=self.request))


@viewlet_config(name='workflow-status',
                context=ISiteManager, layer=IAdminLayer, view=SharedToolAdvancedSearchResultsView,
                manager=IHeaderViewletManager, weight=20)
class SiteManagerSearchResultsStatusViewlet(EmptyViewlet):
    """Site manager advanced search results header viewlet"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyams_content import _
from pyams_content.feature.search.interfaces import ISearchManagerInfo
from pyams_content.interfaces import MANAGE_SITE_ROOT_PERMISSION
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, FormGroupChecker
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuDivider, NavigationMenuItem


@viewlet_config(name='search.divider',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=799,
                permission=MANAGE_SITE_ROOT_PERMISSION)
class SearchManagerMenuDivider(NavigationMenuDivider):
    """Search manager menu divider"""


@viewlet_config(name='search-manager.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=800,
                permission=MANAGE_SITE_ROOT_PERMISSION)
class SearchManagerMenu(NavigationMenuItem):
    """Search manager menu"""

    label = _("Search settings")
    href = '#search-manager.html'


@ajax_form_config(name='search-manager.html',
                  context=ISiteRoot, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_ROOT_PERMISSION)
class SearchManagerPropertiesEditForm(AdminEditForm):
    """Search manager properties edit form"""

    title = _("Search settings")
    legend = _("General search settings")

    fields = Fields(ISearchManagerInfo).select('reference', 'name', 'description')


@adapter_config(name='search-manager-tags-group',
                required=(ISiteRoot, IAdminLayer, SearchManagerPropertiesEditForm),
                provides=IGroup)
class SearchManagerTagsGroup(FormGroupChecker):
    """Search manager tags group"""

    fields = Fields(ISearchManagerInfo).select('enable_tags_search', 'tags_search_target')

    weight = 10


@adapter_config(name='search-manager-collections-group',
                required=(ISiteRoot, IAdminLayer, SearchManagerPropertiesEditForm),
                provides=IGroup)
class SearchManagerCollectionsGroup(FormGroupChecker):
    """Search manager collections group"""

    fields = Fields(ISearchManagerInfo).select('enable_collections_search', 'collections_search_target')

    weight = 20



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.workflow.basic module

This module defines a basic PyAMS content workflow.
"""

from datetime import datetime, timedelta, timezone

from zope.copy import copy
from zope.interface import implementer
from zope.intid.interfaces import IIntIds
from zope.location import locate
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.interfaces import CREATE_VERSION_PERMISSION, MANAGER_ROLE, MANAGE_CONTENT_PERMISSION, \
    MANAGE_SITE_ROOT_PERMISSION, OWNER_ROLE, PILOT_ROLE, PUBLISH_CONTENT_PERMISSION, READER_ROLE, WEBMASTER_ROLE
from pyams_content.shared.common.interfaces import BASIC_CONTENT_WORKFLOW, IContributorRestrictions, \
    IManagerRestrictions, IWfSharedContentRoles
from pyams_content.workflow import CANCELED, ContentArchivingTask, ContentPublishingTask, prepublished_to_published
from pyams_content.workflow.interfaces import IBasicWorkflow
from pyams_scheduler.interfaces import IScheduler
from pyams_scheduler.interfaces.task import IDateTaskScheduling, SCHEDULER_TASK_DATE_MODE
from pyams_security.interfaces import IRoleProtectedObject
from pyams_sequence.interfaces import ISequentialIdInfo
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.date import format_datetime
from pyams_utils.registry import get_current_registry, get_utility, query_utility, utility_config
from pyams_utils.request import check_request
from pyams_utils.timezone import gmtime
from pyams_workflow.interfaces import IWorkflow, IWorkflowInfo, IWorkflowPublicationInfo, IWorkflowState, \
    IWorkflowStateLabel, IWorkflowVersions, ObjectClonedEvent, SYSTEM_TRANSITION
from pyams_workflow.workflow import Transition, Workflow

__docformat__ = 'restructuredtext'

from pyams_content import _


#
# Workflow states
#

DRAFT = 'draft'
PRE_PUBLISHED = 'pre-published'
PUBLISHED = 'published'
ARCHIVED = 'archived'
DELETED = 'deleted'

STATES_IDS = (
    DRAFT,
    PRE_PUBLISHED,
    PUBLISHED,
    ARCHIVED,
    DELETED
)

STATES_LABELS = (
    _("Draft"),
    _("Published (waiting)"),
    _("Published"),
    _("Archived"),
    _("Deleted")
)

STATES_VOCABULARY = SimpleVocabulary([
    SimpleTerm(STATES_IDS[i], title=t)
    for i, t in enumerate(STATES_LABELS)
])

STATES_HEADERS = {
    DRAFT: _("draft created"),
    PRE_PUBLISHED: _("published (waiting)"),
    PUBLISHED: _("published"),
    ARCHIVED: _("archived")
}


UPDATE_STATES = (DRAFT, )
'''Default state available to contributors in update mode'''

READONLY_STATES = (ARCHIVED, DELETED)
'''Retired and archived contents can't be modified'''

PROTECTED_STATES = (PRE_PUBLISHED, PUBLISHED)
'''Protected states are available to webmasters in update mode'''

MANAGER_STATES = ()
'''Only managers can update proposed contents (if their restrictions apply)'''

PUBLISHED_STATES = (PRE_PUBLISHED, PUBLISHED)
'''Contents in published states are pre-published, published or waiting for retiring'''

VISIBLE_STATES = (PUBLISHED, )
'''Contents in visible states are visible in front-office'''

WAITING_STATES = ()
'''Contents in waiting states are waiting for a manager action'''

RETIRED_STATES = ()

ARCHIVED_STATES = (ARCHIVED, )


#
# Workflow conditions
#

def can_create_new_version(wf, context):
    """Check if we can create a new version"""
    # can't create new version when previous draft already exists
    versions = IWorkflowVersions(context)
    if (versions.has_version(DRAFT) or
            versions.has_version(PRE_PUBLISHED)):
        return False
    request = check_request()
    # grant access to webmaster
    if request.has_permission(MANAGE_SITE_ROOT_PERMISSION, context):
        return True
    # grant access to owner, creator and local contributors and managers
    roles = IWfSharedContentRoles(context)
    principal_id = request.principal.id
    if principal_id in {context.creator} | roles.owner | roles.contributors | roles.managers:
        return True
    # grant access to allowed contributors
    restrictions = IContributorRestrictions(context)
    if restrictions and restrictions.can_access(context,
                                                permission=MANAGE_CONTENT_PERMISSION,
                                                request=request):
        return True
    # grant access to shared tool managers if restrictions apply
    restrictions = IManagerRestrictions(context)
    return restrictions and restrictions.can_access(context,
                                                    permission=CREATE_VERSION_PERMISSION,
                                                    request=request)


def can_manage_content(wf, context):
    """Check if a manager can handle content"""
    request = check_request()
    # grant access to webmaster
    if request.has_permission(MANAGE_SITE_ROOT_PERMISSION, context):
        return True
    # local content managers can manage content
    roles = IWfSharedContentRoles(context)
    principal_id = request.principal.id
    if principal_id in roles.owner | roles.managers:
        return True
    # shared tool managers can manage content if restrictions apply
    restrictions = IManagerRestrictions(context)
    return restrictions and restrictions.can_access(context,
                                                    permission=PUBLISH_CONTENT_PERMISSION,
                                                    request=request)


def can_delete_version(wf, context):
    """Check if we can delete a draft version"""
    request = check_request()
    # grant access to webmaster
    if request.has_permission(MANAGE_SITE_ROOT_PERMISSION, context):
        return True
    # grant access to owner, creator and local contributors and managers
    roles = IWfSharedContentRoles(context)
    principal_id = request.principal.id
    if principal_id in {context.creator} | roles.owner | roles.contributors | roles.managers:
        return True
    # grant access to allowed contributors
    restrictions = IContributorRestrictions(context)
    if restrictions and restrictions.can_access(context,
                                                permission=MANAGE_CONTENT_PERMISSION,
                                                request=request):
        return True
    # grant access to shared tool managers if restrictions apply
    restrictions = IManagerRestrictions(context)
    return restrictions and restrictions.can_access(context,
                                                    permission=MANAGE_CONTENT_PERMISSION,
                                                    request=request)


#
# Workflow actions
#

def remove_scheduler_task(context):
    """Remove any scheduler task for this context"""
    scheduler = query_utility(IScheduler)
    if scheduler is not None:
        intids = get_utility(IIntIds)
        context_id = intids.queryId(context)
        task_id = 'workflow::{}'.format(context_id)
        if task_id in scheduler:
            del scheduler[task_id]


def reset_publication_action(wf, context):
    """Refuse version publication"""
    IWorkflowPublicationInfo(context).reset(complete=True)


def prepublish_action(wf, context):
    """Publish content with a future effective publication date

    We create a dedicated publication task which will effectively publish the content
    """
    scheduler = query_utility(IScheduler)
    if scheduler is not None:
        intids = get_utility(IIntIds)
        context_id = intids.queryId(context)
        task_id = 'workflow::{}'.format(context_id)
        if task_id in scheduler:
            del scheduler[task_id]
        task = ContentPublishingTask(context_id,
                                     prepublished_to_published.transition_id)
        task.name = 'Planned publication for {}'.format(ISequentialIdInfo(context).public_oid)
        task.schedule_mode = SCHEDULER_TASK_DATE_MODE
        pub_info = IWorkflowPublicationInfo(context)
        schedule_info = IDateTaskScheduling(task)
        schedule_info.active = True
        schedule_info.start_date = pub_info.publication_effective_date
        scheduler[task_id] = task


def cancel_prepublish_action(wf, context):
    """Cancel pre-publication"""
    remove_scheduler_task(context)


def publish_action(wf, context):
    """Publish version"""
    request = check_request()
    translate = request.localizer.translate
    publication_info = IWorkflowPublicationInfo(context)
    publication_info.publication_date = datetime.now(timezone.utc)
    publication_info.publisher = request.principal.id
    version_id = IWorkflowState(context).version_id
    for version in IWorkflowVersions(context).get_versions((PRE_PUBLISHED, PUBLISHED)):
        if version is not context:
            IWorkflowInfo(version).fire_transition_toward(
                ARCHIVED, comment=translate(_("Published version {0}")).format(version_id))
    # check expiration date and create auto-archiving task if needed
    # we compare expiration date with current date to handle the case where content is
    # published automatically at application startup, and we add a small amount of time
    # to be sure that scheduler and indexer processes are started
    if publication_info.publication_expiration_date:
        scheduler = query_utility(IScheduler)
        if scheduler is not None:
            intids = get_utility(IIntIds)
            context_id = intids.queryId(context)
            task_id = 'workflow::{}'.format(context_id)
            if task_id in scheduler:
                del scheduler[task_id]
            task = ContentArchivingTask(context_id)
            task.name = 'Planned archiving for {}'.format(ISequentialIdInfo(context).public_oid)
            task.schedule_mode = SCHEDULER_TASK_DATE_MODE
            pub_info = IWorkflowPublicationInfo(context)
            now = datetime.now(timezone.utc)
            schedule_info = IDateTaskScheduling(task)
            schedule_info.active = True
            schedule_info.start_date = max(now + timedelta(seconds=10),
                                           pub_info.publication_expiration_date)
            scheduler[task_id] = task


def archive_action(wf, context):
    """Remove readers when a content is archived, and delete any scheduler task"""
    # remove readers
    roles = IWfSharedContentRoles(context, None)
    if roles is not None:
        IRoleProtectedObject(context).revoke_role(READER_ROLE, roles.readers)
    # remove any scheduler task
    remove_scheduler_task(context)


def clone_action(wf, context):
    """Create new version"""
    result = copy(context)
    locate(result, context.__parent__)
    registry = get_current_registry()
    registry.notify(ObjectClonedEvent(result, context))
    return result


def delete_action(wf, context):
    """Delete draft version, and parent if single version"""
    versions = IWorkflowVersions(context)
    versions.remove_version(IWorkflowState(context).version_id)


#
# Workflow transitions
#

init = Transition(transition_id='init',
                  title=_("Initialize"),
                  source=None,
                  destination=DRAFT,
                  history_label=_("Draft creation"))

draft_to_prepublished = Transition(transition_id='draft_to_prepublished',
                                   title=_("Pre-publish content"),
                                   source=DRAFT,
                                   destination=PRE_PUBLISHED,
                                   trigger=SYSTEM_TRANSITION,
                                   action=prepublish_action,
                                   history_label=_("Content pre-published"),
                                   notify_roles={'*'},
                                   notify_title=_("Content publication"),
                                   notify_message=_("{principal} pre-published the content "
                                                    "« {title} »"))

prepublished_to_draft = Transition(transition_id='prepublished_to_draft',
                                   title=_("Cancel publication"),
                                   source=PRE_PUBLISHED,
                                   destination=DRAFT,
                                   permission=MANAGE_CONTENT_PERMISSION,
                                   condition=can_manage_content,
                                   action=cancel_prepublish_action,
                                   menu_icon_class='fas fa-fw fa-reply',
                                   view_name='wf-cancel-publish.html',
                                   history_label=_("Publication canceled"),
                                   notify_roles={WEBMASTER_ROLE, PILOT_ROLE, MANAGER_ROLE, OWNER_ROLE},
                                   notify_title=_("Content publication"),
                                   notify_message=_("{principal} cancelled the publication "
                                                    "for content « {title} »"),
                                   order=1)

prepublished_to_published = Transition(transition_id='prepublished_to_published',
                                       title=_("Publish content"),
                                       source=PRE_PUBLISHED,
                                       destination=PUBLISHED,
                                       trigger=SYSTEM_TRANSITION,
                                       action=publish_action,
                                       history_label=_("Content published"))

draft_to_published = Transition(transition_id='draft_to_published',
                                title=_("Publish content"),
                                source=DRAFT,
                                destination=PUBLISHED,
                                permission=PUBLISH_CONTENT_PERMISSION,
                                condition=can_manage_content,
                                action=publish_action,
                                prepared_transition=draft_to_prepublished,
                                menu_icon_class='fas fa-fw fa-thumbs-up',
                                view_name='wf-publish.html',
                                history_label=_("Content published"),
                                notify_roles={WEBMASTER_ROLE, PILOT_ROLE, MANAGER_ROLE, OWNER_ROLE},
                                notify_title=_("Content publication"),
                                notify_message=_("{principal} published the content "
                                                 "« {title} »"),
                                order=1)

published_to_archived = Transition(transition_id='published_to_archived',
                                   title=_("Archive content"),
                                   source=PUBLISHED,
                                   destination=ARCHIVED,
                                   permission=PUBLISH_CONTENT_PERMISSION,
                                   condition=can_manage_content,
                                   action=archive_action,
                                   menu_icon_class='fas fa-fw fa-archive',
                                   view_name='wf-archive.html',
                                   show_operator_warning=True,
                                   history_label=_("Content archived"),
                                   notify_roles={WEBMASTER_ROLE, PILOT_ROLE, MANAGER_ROLE, OWNER_ROLE},
                                   notify_message=_("{principal} archived content « {title} »"),
                                   order=2)

published_to_draft = Transition(transition_id='published_to_draft',
                                title=_("Create new version"),
                                source=PUBLISHED,
                                destination=DRAFT,
                                permission=CREATE_VERSION_PERMISSION,
                                condition=can_create_new_version,
                                action=clone_action,
                                menu_icon_class='fas fa-fw fa-file',
                                view_name='wf-clone.html',
                                history_label=_("New version created"),
                                order=3)

archived_to_draft = Transition(transition_id='archived_to_draft',
                               title=_("Create new version"),
                               source=ARCHIVED,
                               destination=DRAFT,
                               permission=CREATE_VERSION_PERMISSION,
                               condition=can_create_new_version,
                               action=clone_action,
                               menu_icon_class='fas fa-fw fa-file',
                               view_name='wf-clone.html',
                               history_label=_("New version created"),
                               order=4)

delete = Transition(transition_id='delete',
                    title=_("Delete version"),
                    source=DRAFT,
                    destination=DELETED,
                    permission=MANAGE_CONTENT_PERMISSION,
                    condition=can_delete_version,
                    action=delete_action,
                    menu_icon_class='fas fa-fw fa-trash',
                    view_name='wf-delete.html',
                    history_label=_("Version deleted"),
                    order=99)

wf_transitions = {init,
                  draft_to_prepublished,
                  prepublished_to_draft,
                  prepublished_to_published,
                  draft_to_published,
                  published_to_archived,
                  published_to_draft,
                  archived_to_draft,
                  delete}


@implementer(IBasicWorkflow)
class BasicWorkflow(Workflow):
    """PyAMS basic content workflow"""

    label = _("PyAMS basic workflow")


@adapter_config(required=IBasicWorkflow,
                provides=IWorkflowStateLabel)
class BasicWorkflowStateLabelAdapter(ContextAdapter):
    """Basic workflow state label adapter"""

    @staticmethod
    def get_label(content, request=None, format=True):
        """Workflow state label getter"""
        if request is None:
            request = check_request()
        translate = request.localizer.translate
        state = IWorkflowState(content)
        header = STATES_HEADERS.get(state.state)
        if header is not None:
            state_label = translate(header)
            if format:
                state_label = translate(_('{state} {date}')).format(
                    state=state_label,
                    date=format_datetime(state.state_date))
        else:
            state_label = translate(_("Unknown state"))
        return state_label


@adapter_config(name=DRAFT,
                required=IBasicWorkflow,
                provides=IWorkflowStateLabel)
class BasicWorkflowDraftStateLabelAdapter(ContextAdapter):
    """Basic workflow draft state label adapter"""

    @staticmethod
    def get_label(content, request=None, format=True):
        """Workflow state label getter"""
        if request is None:
            request = check_request()
        translate = request.localizer.translate
        state = IWorkflowState(content)
        if len(state.history) <= 2:
            header = STATES_HEADERS.get(state.state)
            if header is not None:
                if state.version_id == 1:
                    state_label = translate(header)
                else:
                    state_label = translate(_("new version created"))
            else:
                state_label = translate(_("Unknown state"))
        else:
            history_item = state.history[-1]
            if history_item.source_state == CANCELED:
                state_label = translate(_('publication request cancelled'))
            else:
                state_label = translate(_('publication refused'))
        if format:
            state_label = translate(_('{state} {date}')).format(
                state=state_label,
                date=format_datetime(state.state_date))
        return state_label


wf = BasicWorkflow(wf_transitions,
                   states=STATES_VOCABULARY,
                   initial_state=DRAFT,
                   update_states=UPDATE_STATES,
                   readonly_states=READONLY_STATES,
                   protected_states=PROTECTED_STATES,
                   manager_states=MANAGER_STATES,
                   published_states=PUBLISHED_STATES,
                   visible_states=VISIBLE_STATES,
                   waiting_states=WAITING_STATES,
                   retired_states=RETIRED_STATES,
                   archived_states=ARCHIVED_STATES,
                   auto_retired_state=ARCHIVED)


@utility_config(name=BASIC_CONTENT_WORKFLOW, provides=IWorkflow)
class BasicWorkflowUtility:
    """PyAMS basic workflow utility"""

    def __new__(cls):
        return wf



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.skin module

This module defines generic components used to handle paragraphs renderers settings.
"""

from zope.traversing.interfaces import ITraversable

from pyams_content.component.paragraph import IBaseParagraph
from pyams_content.feature.renderer import IRendererSettings
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_workflow.content import HiddenContentPublicationInfo
from pyams_workflow.interfaces import IWorkflowPublicationInfo


__docformat__ = 'restructuredtext'


@adapter_config(name='renderer',
                required=IBaseParagraph,
                provides=ITraversable)
class ParagraphRendererSettingsTraverser(ContextAdapter):
    """Paragraph renderer settings traverser"""

    def traverse(self, name, furtherpath=None):
        """Traverse paragraph to renderer settings"""
        return IRendererSettings(self.context)


@adapter_config(required=IBaseParagraph,
                provides=IWorkflowPublicationInfo)
def base_paragraph_publication_info(context):
    """Base paragraph publication info"""
    if not context.visible:
        return HiddenContentPublicationInfo()
    return None



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.manager module

"""

from pyramid.events import subscriber
from zope.component.interfaces import ISite
from zope.container.folder import Folder
from zope.interface import implementer
from zope.lifecycleevent.interfaces import IObjectAddedEvent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.shared.common import ISharedContent
from pyams_content.shared.common.interfaces import IBaseSharedTool, ISharedTool, \
    ISharedToolContainer
from pyams_i18n.content import I18nManagerMixin
from pyams_security.interfaces import IDefaultProtectionPolicy, IViewContextPermissionChecker
from pyams_security.security import ProtectedObjectMixin
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.factory import factory_config, get_object_factory
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_workflow.interfaces import IWorkflow


__docformat__ = 'restructuredtext'


@factory_config(ISharedToolContainer)
class SharedToolContainer(Folder):
    """Shared tools container"""

    title = FieldProperty(ISharedToolContainer['title'])
    short_name = FieldProperty(ISharedToolContainer['short_name'])


@implementer(IDefaultProtectionPolicy, IBaseSharedTool)
class BaseSharedTool(ProtectedObjectMixin, I18nManagerMixin):
    """Base shared tool class"""

    title = FieldProperty(IBaseSharedTool['title'])
    short_name = FieldProperty(IBaseSharedTool['short_name'])

    shared_content_menu = True
    shared_content_workflow = FieldProperty(IBaseSharedTool['shared_content_workflow'])


@implementer(ISharedTool)
class SharedTool(Folder, BaseSharedTool):
    """Shared tool class"""

    shared_content_type = None
    '''Shared content type must be defined by subclasses'''

    label = FieldProperty(ISharedTool['label'])
    navigation_label = FieldProperty(ISharedTool['navigation_label'])
    facets_label = FieldProperty(ISharedTool['facets_label'])
    facets_type_label = FieldProperty(ISharedTool['facets_type_label'])
    dashboard_label = FieldProperty(ISharedTool['dashboard_label'])

    @property
    def shared_content_factory(self):
        return get_object_factory(ISharedContent, name=self.shared_content_type)


@subscriber(IObjectAddedEvent, context_selector=ISharedTool)
def handle_added_shared_tool(event):
    """Register shared tool after """
    site = get_parent(event.newParent, ISite)
    if site is not None:
        registry = site.getSiteManager()
        if registry is not None:
            tool = event.object
            registry.registerUtility(tool, ISharedTool, name=tool.shared_content_type)


@adapter_config(required=IBaseSharedTool,
                provides=IWorkflow)
def shared_tool_workflow_adapter(context):
    """Shared tool workflow adapter"""
    return query_utility(IWorkflow, name=context.shared_content_workflow)


@adapter_config(required=IBaseSharedTool,
                provides=IViewContextPermissionChecker)
class SharedToolPermissionChecker(ContextAdapter):
    """Shared tool permission checker"""

    edit_permission = MANAGE_TOOL_PERMISSION



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.features.renderer.interfaces module

This module defines base interfaces of content renderers.
"""

from zope.annotation import IAttributeAnnotatable
from zope.contentprovider.interfaces import IContentProvider
from zope.interface import Attribute, Interface


__docformat__ = 'restructuredtext'


RENDERER_SETTINGS_KEY = 'pyams_content.renderer.settings'


class IRenderedContent(IAttributeAnnotatable):
    """Generic interface for any rendered content"""

    renderer = Attribute("Selected renderer name")

    def get_renderer(self, request=None):
        """Get selected renderer implementation"""


class IContentRenderer(IContentProvider):
    """Content renderer interface"""

    label = Attribute("Renderer label")
    weight = Attribute("Renderer weight, used for ordering")

    settings_interface = Attribute("Renderer settings interface")
    resources = Attribute("Iterable of needed Fanstatic resources")

    language = Attribute("Renderer language (if forced)")


class ISharedContentRenderer(IContentRenderer):
    """Shared content renderer interface"""


class IRendererSettings(Interface):
    """Base renderer settings interface"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.portlet.skin.interfaces module

This module defines interfaces of view items portlet renderers settings.
"""

from collections import OrderedDict
from enum import Enum

from zope.interface import Attribute, Interface
from zope.schema import Bool, Choice, Int
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_i18n.schema import I18nTextLineField
from pyams_sequence.interfaces import IInternalReference
from pyams_sequence.schema import InternalReferenceField
from pyams_skin.schema import BootstrapThumbnailsSelectionField

__docformat__ = 'restructuredtext'

from pyams_content import _


class IViewItemTargetURL(Interface):
    """View item target URL"""

    target = Attribute("Reference target")

    url = Attribute("Reference URL")


class HEADER_DISPLAY_MODE(Enum):
    """Header display modes"""
    FULL = 'full'
    START = 'start'
    HIDDEN = 'none'


HEADER_DISPLAY_MODES_NAMES = OrderedDict((
    (HEADER_DISPLAY_MODE.FULL, _("Display full header")),
    (HEADER_DISPLAY_MODE.START, _("Display only header start")),
    (HEADER_DISPLAY_MODE.HIDDEN, _("Hide header"))
), )


HEADER_DISPLAY_MODES_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v.value, title=t)
    for v, t in HEADER_DISPLAY_MODES_NAMES.items()
])


class IViewItemsPortletVerticalRendererSettings(IInternalReference):
    """View items portlet vertical renderer settings interface"""

    display_illustrations = Bool(title=_("Display illustrations?"),
                                 description=_("If 'no', view contents will not display "
                                               "illustrations"),
                                 required=True,
                                 default=True)

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Thumbnails selection"),
        description=_("Selection used to display images thumbnails"),
        default_width={
            'xs': 12,
            'sm': 12,
            'md': 4,
            'lg': 4,
            'xl': 4
        },
        required=False)

    display_breadcrumbs = Bool(title=_("Display breadcrumbs?"),
                               description=_("If 'no', view items breadcrumbs will not be "
                                             "displayed"),
                               required=True,
                               default=True)

    display_tags = Bool(title=_("Display tags?"),
                        description=_("If 'no', view items tags will not be displayed"),
                        required=True,
                        default=True)

    paginate = Bool(title=_("Paginate?"),
                    description=_("If 'no', results pagination will be disabled"),
                    required=True,
                    default=True)

    page_size = Int(title=_("Page size"),
                    description=_("Number of items per page, if pagination is enabled"),
                    required=False,
                    default=10)

    reference = InternalReferenceField(title=_("'See all' link target"),
                                       description=_("Internal reference to site or search "
                                                     "folder displaying full list of view's "
                                                     "contents"),
                                       required=False)

    link_label = I18nTextLineField(title=_("Link label"),
                                   description=_("Label of the link to full list page"),
                                   required=False)


class IViewItemsPortletHorizontalRendererSettings(Interface):
    """View items portlet horizontal renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Thumbnails selection"),
        description=_("Selection used to display images thumbnails"),
        default_width={
            'xs': 3,
            'sm': 3,
            'md': 2,
            'lg': 1,
            'xl': 1
        },
        required=True)


class IViewItemsPortletPanelsRendererSettings(Interface):
    """View items portlet panels renderer settings interface"""

    display_illustrations = Bool(title=_("Display illustrations?"),
                                 description=_("If 'no', view contents will not display "
                                               "illustrations"),
                                 required=True,
                                 default=True)

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Thumbnails selection"),
        description=_("Selection used to display illustrations thumbnails"),
        default_selection='pano',
        default_width={
            'xs': 6,
            'sm': 6,
            'md': 4,
            'lg': 3,
            'xl': 2
        },
        required=True)

    paginate = Bool(title=_("Paginate?"),
                    description=_("If 'no', results pagination will be disabled"),
                    required=True,
                    default=True)

    page_size = Int(title=_("Page size"),
                    description=_("Number of items per page, if pagination is enabled"),
                    required=False,
                    default=9)

    header_display_mode = Choice(title=_("Header display mode"),
                                 description=_("Defines how results headers will be rendered"),
                                 required=True,
                                 vocabulary=HEADER_DISPLAY_MODES_VOCABULARY,
                                 default=HEADER_DISPLAY_MODE.FULL.value)

    start_length = Int(title=_("Start length"),
                       description=_("If you choose to display only header start, you can specify "
                                     "maximum text length"),
                       required=True,
                       default=120)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.container module

This module provides paragraphs containers classes and adapters.
"""

from ZODB.interfaces import IBroken
from pyramid.events import subscriber
from zope.copy import copy
from zope.lifecycleevent import IObjectAddedEvent
from zope.location.interfaces import ISublocations
from zope.traversing.interfaces import ITraversable

from pyams_content.component.paragraph import IParagraphContainerTarget
from pyams_content.component.paragraph.interfaces import IBaseParagraph, IParagraphContainer, \
    IParagraphFactorySettings, IParagraphFactorySettingsTarget, PARAGRAPH_CONTAINER_KEY
from pyams_content.shared.common import IWfSharedContent
from pyams_content.shared.common.interfaces.types import IWfTypedSharedContent
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.container import BTreeOrderedContainer
from pyams_utils.factory import factory_config, get_object_factory
from pyams_utils.traversing import get_parent
from pyams_workflow.interfaces import IWorkflowState

__docformat__ = 'restructuredtext'


@factory_config(IParagraphContainer)
class ParagraphContainer(BTreeOrderedContainer):
    """Paragraph container persistent class"""

    def get_paragraphs(self, factories):
        """Get paragraphs matching given factories"""
        if not isinstance(factories, (list, tuple, set)):
            factories = {factories}
        yield from (
            paragraph
            for paragraph in self.values()
            if paragraph.factory_name in (factories or ())
        )

    def get_visible_paragraphs(self, names=None, anchors_only=False, exclude_anchors=False,
                               factories=None, limit=None):
        """Visible paragraphs getter"""
        count = 0
        if names:
            for name in names:
                paragraph = self.get(name)
                if (paragraph is not None) and \
                        (not IBroken.providedBy(paragraph)) and \
                        paragraph.visible:
                    yield paragraph
                    count += 1
                    if limit and (count == limit):
                        return
        else:
            for paragraph in self.values():
                if IBroken.providedBy(paragraph) or not paragraph.visible:
                    continue
                if anchors_only and not paragraph.anchor:
                    continue
                if exclude_anchors and paragraph.anchor:
                    continue
                if factories and (paragraph.factory_name not in factories):
                    continue
                yield paragraph
                count += 1
                if limit and (count == limit):
                    return


@adapter_config(required=IParagraphContainerTarget,
                provides=IParagraphContainer)
def paragraph_container_factory(target):
    """Paragraphs container factory"""
    return get_annotation_adapter(target, PARAGRAPH_CONTAINER_KEY, IParagraphContainer,
                                  name='++paras++')


@adapter_config(name='paras',
                required=IParagraphContainerTarget,
                provides=ITraversable)
class ParagraphContainerNamespace(ContextAdapter):
    """++paras++ namespace adapter"""

    def traverse(self, name, furtherpath=None):
        """Paragraphs container traverser"""
        target = IParagraphContainer(self.context)
        if name:
            target = target.get(name)
        return target


@adapter_config(name='paras',
                required=IParagraphContainerTarget,
                provides=ISublocations)
class ParagraphContainerSublocations(ContextAdapter):
    """Paragraphs container sub-locations"""

    def sublocations(self):
        """Paragraphs container sub-locations getter"""
        yield from IParagraphContainer(self.context).values()


@subscriber(IObjectAddedEvent, context_selector=IParagraphContainerTarget)
def handle_added_paragraph_container(event):
    """Handle added new paragraphs container

    This subscriber to IObjectAddedEvent is used to automatically
    create new paragraphs based on selected data type or on shared
    content manager settings.
    """
    # don't create new paragraphs if container is not empty
    container = IParagraphContainer(event.object)
    if len(container) > 0:
        return
    content = get_parent(container, IWfSharedContent)
    # only apply manager settings to first version
    state = IWorkflowState(content, None) if content is not None else None
    if (state is not None) and (state.version_id > 1):
        return
    # check for typed shared content settings
    if IWfTypedSharedContent.providedBy(content):
        datatype = content.get_data_type()
        if datatype is not None:
            source = IParagraphContainer(datatype, None)
            if source is not None:
                for paragraph in source.values():
                    container.append(copy(paragraph))
    if len(container) > 0:
        return
    # check for shared content manager auto-created paragraphs
    manager = get_parent(container, IParagraphFactorySettingsTarget)
    if manager is not None:
        settings = IParagraphFactorySettings(manager)
        for name in settings.auto_created_paragraphs or ():
            factory = get_object_factory(IBaseParagraph, name=name)
            if factory is not None:
                container.append(factory())



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.skin.workflow module

This module defines a content provider used for shared content publication date rendering.
"""

from babel import Locale, UnknownLocaleError
from babel.dates import format_date
from zope.interface import Interface

from pyams_i18n.interfaces import INegotiator
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_template.template import template_config
from pyams_utils.registry import get_utility
from pyams_viewlet.viewlet import ViewContentProvider, contentprovider_config
from pyams_workflow.interfaces import IWorkflowPublicationInfo

__docformat__ = 'restructuredtext'


@contentprovider_config(name='pyams_content.publication_date',
                        layer=IPyAMSUserLayer, view=Interface)
@template_config(template='templates/publication-date.pt')
class SharedContentPublicationContentProvider(ViewContentProvider):
    """Shared content publication date content provider"""

    date_format = 'long'

    @property
    def publication_date(self):
        info = IWorkflowPublicationInfo(self.context, None)
        if info is None:
            return ''
        try:
            locale = Locale(self.request.locale_name)
        except UnknownLocaleError:
            negotiator = get_utility(INegotiator)
            locale = Locale(negotiator.server_language)
        return format_date(info.visible_publication_date,
                           format=self.date_format, locale=locale)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.site.zmi.widget.folder module

This module provides a custom widget which can be used to select a folder
inside a site manager.
"""

from zope.interface import implementer_only

from pyams_content.shared.site.zmi.widget.interfaces import ISiteManagerFoldersSelectorWidget
from pyams_form.browser.text import TextWidget
from pyams_form.widget import FieldWidget


__docformat__ = 'restructuredtext'


@implementer_only(ISiteManagerFoldersSelectorWidget)
class SiteManagerFoldersSelectorWidget(TextWidget):
    """Site manager folders selector widget"""

    permission = None


def SiteManagerFoldersSelectorFieldWidget(field, request):  # pylint: disable=invalid-name
    """IFieldWidget factory for TextWidget."""
    return FieldWidget(field, SiteManagerFoldersSelectorWidget(request))



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component module

"""

import json

from pyramid.decorator import reify
from pyramid.httpexceptions import HTTPInternalServerError, HTTPNotFound, HTTPServiceUnavailable
from pyramid.interfaces import IView
from pyramid.view import view_config
from zope.contentprovider.interfaces import IContentProvider
from zope.interface import implementer

from pyams_content.component.association.interfaces import IAssociationContainerTarget
from pyams_content.component.association.zmi.container import AssociationsTableView
from pyams_content.component.paragraph.interfaces import IBaseParagraph, IParagraphContainer, \
    IParagraphContainerTarget
from pyams_content.component.paragraph.zmi.interfaces import IInnerParagraphEditForm, \
    IParagraphContainerBaseTable, IParagraphContainerFullTable, IParagraphContainerView, \
    IParagraphTitleToolbar
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION, PUBLISH_CONTENT_PERMISSION
from pyams_content.shared.common.interfaces.types import ITypedSharedTool
from pyams_content.shared.common.zmi.types.interfaces import ISharedToolTypesTable
from pyams_content.zmi import content_js
from pyams_layer.interfaces import IPyAMSLayer
from pyams_layer.skin import apply_skin
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.viewlet import IContentPrefixViewletManager
from pyams_table.interfaces import IColumn, IValues
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.fanstatic import get_resource_path
from pyams_viewlet.manager import TemplateBasedViewletManager, WeightOrderedViewletManager, \
    viewletmanager_config
from pyams_viewlet.viewlet import ViewContentProvider, viewlet_config
from pyams_zmi.form import AdminModalDisplayForm
from pyams_zmi.helper.container import delete_container_element, switch_element_attribute
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.skin import AdminSkin
from pyams_zmi.table import ActionColumn, AttributeSwitcherColumn, ContentTypeColumn, \
    InnerTableAdminView, MultipleTablesAdminView, NameColumn, ReorderColumn, Table, \
    TableAdminView, TrashColumn, VisibilityColumn, get_ordered_data_attributes, get_table_id
from pyams_zmi.utils import get_object_hint, get_object_icon, get_object_label
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(IParagraphContainerBaseTable)
class ParagraphsBaseTable(Table):
    """Paragraphs container table"""

    @reify
    def data_attributes(self):
        """Attributes getter"""
        attributes = super().data_attributes
        attributes.setdefault('table', {}).update({
            'data-ams-modules': json.dumps({
                'content': {
                    'src': get_resource_path(content_js)
                }
            })
        })
        container = IParagraphContainer(self.context)
        get_ordered_data_attributes(self, attributes, container, self.request)
        return attributes

    display_if_empty = True


@factory_config(IParagraphContainerFullTable)
@implementer(IView)
class ParagraphsTable(ParagraphsBaseTable):
    """Full paragraphs table"""


@adapter_config(required=(IParagraphContainer, IAdminLayer, IParagraphContainerBaseTable),
                provides=IValues)
class ParagraphContainerValues(ContextRequestViewAdapter):
    """Paragraph container values adapter"""

    @property
    def values(self):
        """Paragraph container values getter"""
        yield from IParagraphContainer(self.context).values()


@adapter_config(name='reorder',
                required=(IParagraphContainer, IAdminLayer, IParagraphContainerBaseTable),
                provides=IColumn)
class ParagraphsReorderColumn(ReorderColumn):
    """Paragraphs reorder column"""


@view_config(name='reorder.json',
             context=IParagraphContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def reorder_paragraphs_table(request):
    """Reorder paragraphs"""
    order = request.params.get('order').split(';')
    request.context.updateOrder(order)
    return {
        'status': 'success',
        'closeForm': False
    }


@adapter_config(name='visible',
                required=(IParagraphContainer, IAdminLayer, IParagraphContainerBaseTable),
                provides=IColumn)
class ParagraphsVisibleColumn(VisibilityColumn):
    """Paragraphs table visible column"""


@view_config(name='switch-visible-item.json',
             context=IParagraphContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def switch_visible_item(request):
    """Switch visible item"""
    return switch_element_attribute(request)


@adapter_config(name='anchor',
                required=(IParagraphContainer, IAdminLayer, IParagraphContainerBaseTable),
                provides=IColumn)
class ParagraphsAnchorColumn(AttributeSwitcherColumn):
    """Paragraphs table anchor column"""

    attribute_name = 'anchor'
    attribute_switcher = 'switch-anchor-item.json'

    icon_on_class = 'fas fa-anchor'
    icon_off_class = 'fas fa-anchor opacity-25'

    weight = 2

    def get_icon_hint(self, item):
        """Icon hint getter"""
        if self.has_permission(item):
            hint = _("Click to set/unset paragraph as navigation anchor")
        elif item.anchor:
            hint = _("This paragraph is defined as a navigation anchor")
        else:
            hint = _("This paragraph is not defined as a navigation anchor")
        return self.request.localizer.translate(hint)


@view_config(name='switch-anchor-item.json',
             context=IParagraphContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def switch_anchor_item(request):
    """Switch anchor item"""
    return switch_element_attribute(request)


@adapter_config(name='icon',
                required=(IParagraphContainer, IAdminLayer, IParagraphContainerBaseTable),
                provides=IColumn)
class ParagraphsIconColumn(ContentTypeColumn):
    """Paragraphs table icon column"""


@viewletmanager_config(name='pyams_content.paragraph.title-toolbar',
                       context=IBaseParagraph, layer=IAdminLayer,
                       view=IParagraphContainerBaseTable,
                       provides=IParagraphTitleToolbar)
@template_config(template='templates/title-toolbar.pt', layer=IAdminLayer)
class ParagraphTitleToolbar(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """Paragraph title toolbar viewlet manager"""

    render_empty = True

    @property
    def id(self):
        """Unique ID getter"""
        return f'{get_table_id(self.view)}_title_{self.context.__name__}'


class ParagraphTitleToolbarItemMixin:
    """Paragraph title toolbar item"""

    icon_hint = None
    icon_class = None

    target_intf = None
    item_intf = None

    counter = 0

    def update(self):
        target = self.target_intf(self.context, None)
        if target is None:
            return
        self.counter = sum(1 for _ in filter(self.item_intf.providedBy, target.values()))

    def checker(self):
        """Context checker"""
        return self.counter > 0

    def render(self):
        """Viewlet render"""
        if not self.checker():
            return ''
        translate = self.request.localizer.translate
        return f'<span class="px-2">' \
               f'  {self.counter or ""} ' \
               f'  <i class="{self.icon_class} hint pl-1" ' \
               f'     data-original-title="{translate(self.icon_hint)}"></i>' \
               f'</span>'


@adapter_config(name='label',
                required=(IParagraphContainer, IAdminLayer, IParagraphContainerBaseTable),
                provides=IColumn)
class ParagraphsLabelColumn(NameColumn):
    """Paragraphs label column"""

    i18n_header = _("Title")

    css_classes = {
        'td': 'title'
    }


@adapter_config(name='label',
                required=(IParagraphContainer, IAdminLayer, IParagraphContainerFullTable),
                provides=IColumn)
class ParagraphsFullLabelColumn(ParagraphsLabelColumn):
    """Paragraphs label column"""

    i18n_header = _("Show/hide all paragraphs")
    head_hint = _("Click to show/hide all paragraphs editors")
    cell_hint = _("Click to show/hide paragraph editor")

    css_classes = {}

    def render_head_cell(self):
        """Head cell renderer"""
        hint = self.request.localizer.translate(self.head_hint)
        return f'<span data-ams-click-handler="MyAMS.content.paragraphs.switchAllEditors" ' \
               f'      data-ams-stop-propagation="true">' \
               f'  <span class="switcher mr-2 hint" data-original-title="{hint}">' \
               f'    <i class="far fa-plus-square"></i>' \
               f'  </span>' \
               f'  <span class="title">{super().render_head_cell()}</span>' \
               f'</span>'

    def render_cell(self, item):
        """Cell renderer"""
        toolbar = ''
        provider = self.request.registry.queryMultiAdapter(
            (item, self.request, self.table), IContentProvider,
            name='pyams_content.paragraph.title-toolbar')
        if provider is not None:
            provider.update()
            toolbar = provider.render()
        hint = self.request.localizer.translate(self.cell_hint)
        return f'<div class="switcher-parent" ' \
               f'     data-ams-click-handler="MyAMS.content.paragraphs.switchEditor" ' \
               f'     data-ams-stop-propagation="true">' \
               f'  <span class="switcher mr-2 hint" data-original-title="{hint}">' \
               f'    <i class="far fa-plus-square"></i>' \
               f'  </span>' \
               f'  <span class="title">{super().render_cell(item)}</span>' \
               f'  {toolbar}' \
               f'</div>' \
               f'<div class="editor"></div>'


@adapter_config(name='lock',
                required=(IParagraphContainer, IAdminLayer, IParagraphContainerBaseTable),
                provides=IColumn)
class ParagraphsLockColumn(AttributeSwitcherColumn):
    """Paragraphs lock column"""

    permission = PUBLISH_CONTENT_PERMISSION

    attribute_name = 'locked'
    attribute_switcher = 'switch-paragraph-lock.json'

    icon_on_class = 'fas fa-lock'
    icon_off_class = 'fas fa-unlock opacity-50'

    weight = 900

    def get_icon_hint(self, item):
        """Icon hint getter"""
        if self.has_permission(item):
            hint = _("Click to lock/unlock paragraph")
        elif item.locked:
            hint = _("This paragraph is locked and can't be removed!")
        else:
            hint = _("This paragraph is not locked")
        return self.request.localizer.translate(hint)


@view_config(name='switch-paragraph-lock.json',
             context=IParagraphContainer, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION,
             renderer='json', xhr=True)
def switch_paragraph_lock(request):
    """Switch paragraph lock"""
    return switch_element_attribute(request)


@adapter_config(name='trash',
                required=(IParagraphContainer, IAdminLayer, IParagraphContainerBaseTable),
                provides=IColumn)
class ParagraphsTrashColumn(TrashColumn):
    """Paragraphs table trash column"""


@view_config(name='delete-element.json',
             context=IParagraphContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def delete_data_type(request):
    """Delete data type"""
    return delete_container_element(request)


#
# Paragraphs table and views
#

@viewlet_config(name='paragraphs.menu',
                context=IParagraphContainerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=100,
                permission=VIEW_SYSTEM_PERMISSION)
class ParagraphsMenu(NavigationMenuItem):
    """Paragraphs menu"""

    label = _("Paragraphs")
    href = '#paragraphs.html'


class ParagraphsContainerViewMixin:
    """Paragraphs container view mixin class"""

    table_class = IParagraphContainerBaseTable
    table_label = _("Paragraphs list")

    container_intf = IParagraphContainer


@pagelet_config(name='paragraphs.html',
                context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
@implementer(IParagraphContainerView)
class ParagraphsContainerView(ParagraphsContainerViewMixin, TableAdminView):
    """Paragraphs container view"""

    table_class = IParagraphContainerFullTable


@adapter_config(name='paragraphs',
                required=(ITypedSharedTool, IAdminLayer, ISharedToolTypesTable),
                provides=IColumn)
class SharedToolTypesParagraphsColumn(ActionColumn):
    """Shared tool data types table paragraphs column"""

    hint = _("Default paragraphs")
    icon_class = 'fas fa-paragraph'

    href = 'paragraphs-modal.html'
    modal_target = True

    weight = 400


@pagelet_config(name='paragraphs-modal.html',
                context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class ParagraphsContainerModalView(AdminModalDisplayForm):
    """Paragraphs container modal view"""

    subtitle = _("Paragraphs")
    modal_class = 'modal-xl'
    modal_content_class = 'min-height-50vh'


@adapter_config(required=(IParagraphContainerTarget, IAdminLayer, ParagraphsContainerModalView),
                provides=IFormTitle)
def paragraph_container_modal_view_title(context, request, view):
    """Paragraph container modal view title"""
    hint = get_object_hint(context, request, view)
    label = get_object_label(context, request, view)
    return TITLE_SPAN_BREAK.format(hint, label)


@viewlet_config(name='paragraphs-modal-table',
                context=IParagraphContainerTarget, layer=IAdminLayer,
                view=ParagraphsContainerModalView,
                manager=IContentPrefixViewletManager, weight=10)
class ParagraphsContainerTableView(ParagraphsContainerViewMixin, InnerTableAdminView):
    """Paragraphs container table view"""


@view_config(name='get-paragraphs-editors.json',
             context=IParagraphContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=VIEW_SYSTEM_PERMISSION)
def get_paragraphs_editors(request):
    """Get internal editors of all paragraphs"""
    apply_skin(request, AdminSkin)
    result = {}
    for name, element in IParagraphContainer(request.context).items():
        editor = request.registry.queryMultiAdapter((element, request), IInnerParagraphEditForm)
        if editor is None:
            raise HTTPInternalServerError("Can't get inner paragraphs editors!")
        editor.update()
        result[name] = editor.render()
    return result


@view_config(name='get-paragraph-editor.json',
             context=IParagraphContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=VIEW_SYSTEM_PERMISSION)
def get_paragraph_editor(request):
    """Get internal editor of a paragraph"""
    name = request.params.get('object_name')
    if not name:
        raise HTTPServiceUnavailable()
    element = IParagraphContainer(request.context).get(name)
    if element is None:
        raise HTTPNotFound()
    apply_skin(request, AdminSkin)
    editor = request.registry.queryMultiAdapter((element, request), IInnerParagraphEditForm)
    if editor is None:
        raise HTTPInternalServerError("Can't get inner paragraph editor!")
    editor.update()
    return {
        element.__name__: editor.render()
    }


#
# Paragraphs associations view components
#

@viewlet_config(name='paragraphs-associations.menu',
                context=IParagraphContainerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=110,
                permission=VIEW_SYSTEM_PERMISSION)
class ParagraphsAssociationsMenu(NavigationMenuItem):
    """Paragraphs associations menu"""

    label = _("Links and attachments")
    href = '#paragraphs-associations.html'


class ParagraphAssociationsTableView(AssociationsTableView, ViewContentProvider):
    """Paragraph associations table view"""

    @property
    def table_label(self):
        """Table label getter"""
        return f'<i class="fa-fw {get_object_icon(self.context, self.request)} hint mr-1"' \
               f'   data-original-title="{get_object_hint(self.context, self.request)}"></i> ' \
               f'{get_object_label(self.context, self.request)}'


@pagelet_config(name='paragraphs-associations.html',
                context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class ParagraphsAssociationsView(MultipleTablesAdminView):
    """Paragraphs associations view"""

    table_label = _("Content blocks links and attachments")

    @reify
    def tables(self):
        """Tables getter"""
        result = []
        container = IParagraphContainer(self.context)
        for paragraph in filter(IAssociationContainerTarget.providedBy, container.values()):
            result.append(ParagraphAssociationsTableView(paragraph, self.request, self))
        return result



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.workflow.notify module

This module defines components which are used to send notifications to managers on
selected workflow events.
"""

from pyramid.events import subscriber
from pyramid.location import lineage

from pyams_content.interfaces import MANAGER_ROLE, MANAGE_CONTENT_PERMISSION
from pyams_content.shared.common.interfaces import IManagerRestrictions, \
    IManagerWorkflowRestrictions
from pyams_i18n.interfaces import II18n
from pyams_security.interfaces import IProtectedObject
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.request import query_request
from pyams_utils.url import absolute_url
from pyams_workflow.interfaces import IWorkflowTransitionEvent


__docformat__ = 'restructuredtext'


try:
    from pyams_chat.message import ChatMessage
    from pyams_chat.interfaces import IChatMessage, IChatMessageHandler
except ImportError:
    pass
else:

    @subscriber(IWorkflowTransitionEvent)
    def handle_workflow_transition(event):
        """Handle workflow transition event"""
        request = query_request()
        if request is None:
            return
        transition = event.transition
        if not (transition.user_data.get('notify_roles') and
                transition.user_data.get('notify_message')):
            return
        content = event.object
        translate = request.localizer.translate
        title = II18n(content).query_attribute('title', request=request)
        message = ChatMessage(request=request,
                              context=content,
                              action='notify',
                              category='content.workflow',
                              source=request.principal,
                              title=translate(transition.user_data.get('notify_title',
                                                                       transition.title)),
                              message=translate(transition.user_data.get('notify_message',
                                                                         '')).format(
                                  principal=request.principal.title,
                                  title=title),
                              url=absolute_url(content, request, 'admin'),
                              comment=event.comment,
                              transition=transition)
        message.send()


    @adapter_config(name='content.workflow',
                    required=IChatMessage,
                    provides=IChatMessageHandler)
    class ContentWorkflowTransitionChatHandler(ContextAdapter):
        """Content workflow transition chat message handler"""

        def get_target(self):
            """Chat message targets getter"""
            notify_roles = self.context.user_data['transition'].user_data.get('notify_roles', ())
            if '*' in notify_roles:
                return {}
            request = query_request()
            notification_source = self.context.context
            principals = set()
            for context in lineage(notification_source):
                protection = IProtectedObject(context, None)
                if protection is None:
                    continue
                for role_id in notify_roles:
                    if role_id == MANAGER_ROLE:
                        restrictions = IManagerRestrictions(context, None)
                        if restrictions is None:
                            continue
                        for principal in protection.get_principals(role_id):
                            principal_restrictions = restrictions.get_restrictions(principal)
                            if principal_restrictions:
                                workflow_restrictions = \
                                    IManagerWorkflowRestrictions(principal_restrictions, None)
                                if workflow_restrictions.can_access(notification_source,
                                                                    MANAGE_CONTENT_PERMISSION,
                                                                    request=request):
                                    principals.add(principal)
                    else:
                        principals |= protection.get_principals(role_id)
            source_id = self.context.source['id']
            if source_id in principals:
                principals.remove(source_id)
            return {
                'principals': tuple(principals)
            }
